package de.unimarburg.swt.emf.fix.model;

import java.util.Map;
import java.util.Set;

import org.eclipse.emf.refactor.common.core.EmfRefactoring;

import de.unimarburg.swt.emf.fix.util.RelationLoader;
import de.unimarburg.swt.emf.modelsmell.model.ModelSmell;

/**
 * Singleton class wrapping the relation between model refactorings and possibly
 * caused model smells provided by extensions of EMF Fix'es extension point.
 * 
 * @author Matthias Burhenne
 */
public class RefactoringToSmellsRelation {
	public static final RefactoringToSmellsRelation INSTANCE = new RefactoringToSmellsRelation();
	private Map<EmfRefactoring, Set<ModelSmell>> relationMap;
	
	private RefactoringToSmellsRelation(){
		relationMap = RelationLoader.loadRefactoringToSmellsRelation();
	}
	
	/**
	 * Return all the potentially caused smells for the given EmfRefactoring.
	 * @param refactoring - the refactoring that is potentially causing the smells
	 * @return - Set of ModelSmell objects
	 */
	public Set<ModelSmell> getCausedSmells(EmfRefactoring refactoring){
		return relationMap.get(refactoring);
	}
	
	
}
