package de.unimarburg.swt.emf.fix.model;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.eclipse.emf.refactor.common.core.EmfRefactoring;

import de.unimarburg.swt.emf.fix.util.RelationLoader;
import de.unimarburg.swt.emf.modelsmell.model.ModelSmell;

/**
 * Singleton class wrapping the relation between model smells and possibly
 * fixing model refactorings provided by extensions of EMF Fix'es extension point.
 * 
 * @Author Matthias Burhenne
 */
public class SmellToRefactoringsRelation {
	public static final SmellToRefactoringsRelation INSTANCE = new SmellToRefactoringsRelation();
	private Map<ModelSmell, Set<EmfRefactoring>> relationMap;
	
	private SmellToRefactoringsRelation(){
		relationMap = RelationLoader.loadSmellToRefactoringsRelation();
	}
	
	/**
	 * Returns the model refactorings potentially fixing the given ModelSmell.
	 * 
	 * @param smell -  the model smell that is to be fixed
	 * @return Set of EmfRefactoring objects
	 */
	public Set<EmfRefactoring> getFixingRefactorings(ModelSmell smell){
		Set<EmfRefactoring> result = relationMap.get(smell);
		if(result != null)
			return relationMap.get(smell);
		else
			return new HashSet<EmfRefactoring>();
	}
	
}
