package de.unimarburg.swt.emf.fix.views.contribution;

import java.util.ArrayList;

import org.eclipse.core.resources.IFile;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.refactor.common.config.EmfRefactorConfig;
import org.eclipse.emf.refactor.common.core.EmfRefactoring;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ltk.ui.refactoring.RefactoringWizard;
import org.eclipse.ltk.ui.refactoring.RefactoringWizardOpenOperation;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

import de.unimarburg.swt.emf.fix.EMFFix;
import de.unimarburg.swt.emf.fix.dialogs.SuggestionDialog;
import de.unimarburg.swt.emf.modelsmell.model.ModelSmell;
import de.unimarburg.swt.emf.modelsmell.model.smellfinder.treeviewer.EObjectGroup;

/**
 * Action class for the execution of the suggestion making process.
 * 
 * @author Matthias Burhenne
 *
 */
public class SuggestRefactoringsAction implements IObjectActionDelegate {

	private Shell shell;
	private ISelection selection;
	
	/**
	 * Constructor for Action1.
	 */
	public SuggestRefactoringsAction() {
		super();
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
		shell = targetPart.getSite().getShell();
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		EObjectGroup group = null;
		if(selection instanceof IStructuredSelection){
			IStructuredSelection structSelect = (IStructuredSelection)selection;
			if(!structSelect.isEmpty() && structSelect.getFirstElement() instanceof EObjectGroup)
				group = (EObjectGroup) structSelect.getFirstElement();
		}
		if(group != null){
			ModelSmell smell = group.getModelSmell();
			SuggestionDialog dialog = new SuggestionDialog(shell, EMFFix.getCausedModelSmells(EMFFix.getFixingRefactorings(smell)), group);
			int dialogResult = dialog.open();
			if(dialogResult == Dialog.OK){
				EmfRefactoring selectedRefactoring = dialog.getSelectedRefactoring();
				EObject contextObject = dialog.getSelectedContextObject();
				IFile iFile = group.getModelSmellResult().getResultModel().getIFile();
				EmfRefactorConfig.getInstance().loadProjectGroupMappings(iFile.getProject());
				ArrayList<EObject> contextList = new ArrayList<EObject>();
				contextList.add(contextObject);
				selectedRefactoring.getController().setSelection(contextList);
				selectedRefactoring.getController().getDataManagementObject().preselect(contextList);
				RefactoringWizard wizard = selectedRefactoring.getGui().show();
				RefactoringWizardOpenOperation wizardDialog = new RefactoringWizardOpenOperation(wizard);
				try {
					wizardDialog.run(shell, "EMF-Fix");
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
			

		}
	}

	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		this.selection = selection;
	}

}
