package de.unimarburg.swt.emf.metrics.calculator.henshin;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.henshin.interpreter.EmfEngine;
import org.eclipse.emf.henshin.interpreter.RuleApplication;
import org.eclipse.emf.henshin.interpreter.UnitApplication;
import org.eclipse.emf.henshin.interpreter.interfaces.InterpreterEngine;
import org.eclipse.emf.henshin.interpreter.util.Match;
import org.eclipse.emf.henshin.interpreter.util.ModelHelper;
import org.eclipse.emf.henshin.matching.EmfGraph;
import org.eclipse.emf.henshin.model.Parameter;
import org.eclipse.emf.henshin.model.Rule;
import org.eclipse.emf.henshin.model.TransformationSystem;
import org.eclipse.emf.henshin.model.TransformationUnit;

import de.unimarburg.swt.emf.metrics.model.HenshinTransformationsPack;

public class HenshinUtils {

	private final static String MAIN_UNIT_TAG = "mainUnit";
	private final static String MAIN_RULE_TAG = "mainRule";
	private final static String SELECTED_EOBJECT_TAG = "selectedEObject";

	public static double run(HenshinTransformationsPack pack, List<EObject> context) {
		if (!pack.getPrePath().isEmpty()) {
			process(pack.getPrePath(), context);
		}
		double result = count(pack.getTransformationPath(), context);
		if (!pack.getPostPath().isEmpty()) {
			process(pack.getPostPath(), context);
		}
		return result;
	}

	private static void process(String transf, List<EObject> context) {
		EObject root = EcoreUtil.getRootContainer(context.get(0));
		EmfGraph graph = new EmfGraph();
		graph.addRoot(root);
		InterpreterEngine engine = new EmfEngine(graph);
		TransformationSystem transformationSystem = (TransformationSystem) ModelHelper.loadFile(transf);
		TransformationUnit mainUnit = transformationSystem.findUnitByName(MAIN_UNIT_TAG);
		Map<Parameter, Object> values = new HashMap<Parameter, Object>();
		System.out.println(mainUnit.getName());
		for (Parameter par : mainUnit.getParameters()) {
			if (par.getName().equals(SELECTED_EOBJECT_TAG)) {
				values.put(par, context.get(0));
			}
		}
		System.out.println(values.size());
		UnitApplication application = new UnitApplication(engine, mainUnit,	values);
		
		application.execute();
		System.out.println(application.getAppliedRules().size());
	}

	private static double count(String transf, List<EObject> context) {
		EObject root = EcoreUtil.getRootContainer(context.get(0));
		EmfGraph graph = new EmfGraph();
		graph.addRoot(root);
		InterpreterEngine engine = new EmfEngine(graph);
		TransformationSystem transformationSystem = (TransformationSystem) ModelHelper.loadFile(transf);
		Rule rule = transformationSystem.findRuleByName(MAIN_RULE_TAG);
		RuleApplication application = new RuleApplication(engine, rule);
		Map<Parameter, Object> values = new HashMap<Parameter, Object>();
		for (Parameter par : rule.getParameters()) {
			if (par.getName().equals(SELECTED_EOBJECT_TAG)) {
				values.put(par, context.get(0));
			}
		}
		Match match = new Match(rule, values, ModelHelper.createPrematch(rule,values));
		application.setMatch(match);
		List<Match> matches = application.findAllMatches();
		return matches.size();
	}

//	public static String isCompatible(String transf){
//	TransformationSystem transformationSystem = (TransformationSystem) ModelHelper.loadFile(transf);
//	Rule rule = transformationSystem.findRuleByName(MAIN_RULE_TAG);
//	for (Parameter par : rule.getParameters()) {
//		if (par.getName().equals(SELECTED_EOBJECT_TAG))
//			return "rule";
//	}
//	TransformationUnit mainUnit = transformationSystem.findUnitByName(MAIN_UNIT_TAG);
//	if(mainUnit!=null)
//		for (Parameter par : mainUnit.getParameters()) {
//			if (par.getName().equals(SELECTED_EOBJECT_TAG)) 
//				return "unit";
//	}
//	return "";
//}
	
}
