package de.unimarburg.swt.emf.metrics.gui.view;

import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.logging.Level;

import org.eclipse.birt.core.framework.Platform;
import org.eclipse.birt.report.engine.api.EngineConfig;
import org.eclipse.birt.report.engine.api.EngineConstants;
import org.eclipse.birt.report.engine.api.EngineException;
import org.eclipse.birt.report.engine.api.IGetParameterDefinitionTask;
import org.eclipse.birt.report.engine.api.IReportEngine;
import org.eclipse.birt.report.engine.api.IReportEngineFactory;
import org.eclipse.birt.report.engine.api.IReportRunnable;
import org.eclipse.birt.report.engine.api.IRunAndRenderTask;
import org.eclipse.birt.report.engine.api.PDFRenderOption;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;
import org.osgi.framework.Bundle;

import de.unimarburg.swt.emf.metrics.Activator;
import de.unimarburg.swt.emf.metrics.EMFMetrics;
import de.unimarburg.swt.emf.metrics.birt.MetricsReporter;
import de.unimarburg.swt.emf.metrics.birt.ReportingWizard;
import de.unimarburg.swt.emf.metrics.birt.ReportingWizardDialog;
import de.unimarburg.swt.emf.metrics.model.calculator.Result;
import de.unimarburg.swt.emf.metrics.utils.xml.XMLResultsFile;

public class ResultsView extends ViewPart {

	private static final String RESULT_COLUMN_LABEL = "Result";

	private static final String DESCRIPTION_COLUMN_LABEL = "Description";

	private static final String METRIC_COLUMN_LABEL = "Metric";

	private static final String CONTEXT_COLUMN_LABEL = "Context";

	private static final String TIME_COLUMN_LABEL = "Time";
	
	private static final  String DESIGN_DIR  = "/designs";
	
	private static final String OUTPUT_DIR = "/output";
	
	private static final  String REPORT_FILE_EXTENSION = ".rptdesign";
	
	private static final String DOCUMENT_FILE_EXTENSION = ".rptdocument";
	
	private static final String HTML_FILE_EXTENSION = ".html";
	
	private static final String PDF_FILE_EXTENSION = ".pdf";

	/**
	 * The ID of the view as specified by the extension.
	 */
	public static final String ID = "de.unimarburg.swt.emf.metrics.views.resultsView";

	private TableViewer viewer;
	private Composite parent;
	private SaveAction saveAction;
	private Action clearAction;
	private ExportAction exportAction;

	public ResultsView() {

	}

	public void setFocus() {
		viewer.getControl().setFocus();
	}

	public void createPartControl(Composite parent) {
		this.parent = parent;
		TableColumn column;
		viewer = new TableViewer(parent, SWT.MULTI | SWT.H_SCROLL
				| SWT.V_SCROLL);
		final Table table = viewer.getTable();
		EMFMetrics.setResultsViewer(viewer);
		column = new TableColumn(table, SWT.LEFT);
		column.setText(TIME_COLUMN_LABEL);
		column.setWidth(150);
		column.addListener(SWT.Selection, new ColumnSortListener(column));
		column = new TableColumn(table, SWT.LEFT);
		column.setText(CONTEXT_COLUMN_LABEL);
		column.setWidth(200);
		column.addListener(SWT.Selection, new ColumnSortListener(column));
		column = new TableColumn(table, SWT.LEFT);
		column.setText(METRIC_COLUMN_LABEL);
		column.setWidth(200);
		column.addListener(SWT.Selection, new ColumnSortListener(column));
		column = new TableColumn(table, SWT.LEFT);
		column.setText(DESCRIPTION_COLUMN_LABEL);
		column.setWidth(300);
		column.addListener(SWT.Selection, new ColumnSortListener(column));
		column = new TableColumn(table, SWT.LEFT);
		column.setText(RESULT_COLUMN_LABEL);
		column.setWidth(50);
		column.addListener(SWT.Selection, new ColumnSortListener(column));
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
		viewer.setContentProvider(new ResultsViewContentProvider());
		viewer.setLabelProvider(new ResultsViewLabelProvider());
		viewer.setInput(EMFMetrics.getResultsViewInput());
		PlatformUI.getWorkbench().getHelpSystem().setHelp(viewer.getControl(),"EMFMetrics.viewer");
		makeActions();
		hookContextMenu();
		//hookDoubleClickAction();
		contributeToActionBars();
	}

	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				ResultsView.this.fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}

	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}

	private void fillLocalPullDown(IMenuManager manager) {
		manager.add(saveAction);
		manager.add(exportAction);
		manager.add(clearAction);
		manager.add(new Separator());
	}

	private void fillContextMenu(IMenuManager manager) {
		manager.add(saveAction);
		manager.add(exportAction);
		manager.add(clearAction);
		// Other plug-ins can contribute there actions here
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	private void fillLocalToolBar(IToolBarManager manager) {
		manager.add(saveAction);
		manager.add(exportAction);
		manager.add(clearAction);
	}

	private void makeActions() {
		saveAction = new SaveAction(parent.getShell());
		saveAction.setText("Save Results");
		saveAction.setToolTipText("Save the results list to a file");
		saveAction.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(ISharedImages.IMG_ETOOL_SAVE_EDIT));
		
		clearAction = new Action(){
			public void run() {
				((ResultsViewContentProvider)viewer.getContentProvider()).removeAll();
			}
		};
		clearAction.setText("Clear Results");
		clearAction.setToolTipText("Remove all results from the list");
		clearAction.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(ISharedImages.IMG_ETOOL_DELETE));
		
		exportAction = new ExportAction(parent.getShell());
		exportAction.setText("Export Results");
		exportAction.setToolTipText("Export the results list in various reports");
		exportAction.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(ISharedImages.IMG_ETOOL_PRINT_EDIT));
		
	}
	
	
	private class ExportAction extends Action {
		
		Shell shell;

		ExportAction(Shell shell) {
			super();
			this.shell = shell;
		}
		
		@SuppressWarnings("unchecked")
		public void run() {
//			todo();
			MetricsReporter metricsReporter = new MetricsReporter(shell, (List<Result>) viewer.getInput());			
//			MessageDialog.openInformation(shell, "BIRT Export", "Export Action");
			ReportingWizard wizard = new ReportingWizard(metricsReporter);
			ReportingWizardDialog dialog = new ReportingWizardDialog(shell, wizard);
			dialog.create();
			dialog.open();
		}

		@SuppressWarnings("unchecked")
		private void todo() {
			// configure BIRT engine
			EngineConfig config = new EngineConfig();
			config.setLogConfig("D:/tmp", Level.FINE);
			IReportEngineFactory factory = (IReportEngineFactory) Platform
					.createFactoryObject(IReportEngineFactory.EXTENSION_REPORT_ENGINE_FACTORY);
			// create BIRT engine
			IReportEngine engine = factory.createReportEngine(config);
			engine.changeLogLevel(Level.WARNING);
			IReportRunnable design;
			try {
				// load BIRT report
				String designFileName = getDesignDirectory() + "List" + REPORT_FILE_EXTENSION;
				System.out.println("designFileName: " + designFileName);
				design = engine.openReportDesign(designFileName);				
				// load BIRT run and render task
				IRunAndRenderTask runAndRenderTask = engine.createRunAndRenderTask(design);
				runAndRenderTask.getAppContext().put(EngineConstants.APPCONTEXT_CLASSLOADER_KEY, ResultsView.class.getClassLoader());
								
				// Parameter passing
				IGetParameterDefinitionTask paramTask = engine.createGetParameterDefinitionTask(design);
//				Collection params = paramTask.getParameterDefns(true);
//				Iterator iter = params.iterator();
//				while (iter.hasNext()) {
//					IParameterDefnBase param = (IParameterDefnBase) iter.next();
//					if (param instanceof IParameterGroupDefn) {
//						IParameterGroupDefn group = (IParameterGroupDefn) param;
//						System.out.println("Group Parameter: " + group.getName());
//					} else {
//						IScalarParameterDefn scalar = (IScalarParameterDefn) param;
//						System.out.println("Scalar Parameter: " + scalar.getName());
//					}
//				}
				System.out.println("Parameter XmlUrl vorher: " + paramTask.getParameterValue("XmlUrl"));
//				paramTask.setParameterValue("XmlUrl", "C:\\Users\\arendt.INFORMATIK\\Desktop\\test.xml");
//				paramTask.setValue("XmlUrl", "C:\\Users\\arendt.INFORMATIK\\Desktop\\test2.xml");
				paramTask.setValue("XmlUrl", "C:/Users/arendt.INFORMATIK.001/Desktop/test2.xml");
				System.out.println("Parameter XmlUrl nachher: " + paramTask.getParameterValue("XmlUrl"));
				HashMap<String, Object> parameterValues = paramTask.getParameterValues( );
//				for (String key : parameterValues.keySet()) {
//					System.out.println("Key/Value in parameterValues: " + key + " / " + parameterValues.get(key));
//				}
				// Set values for all parameters in a HashMap, parameterValues
				runAndRenderTask.setParameterValues( parameterValues );
				// Validate parameter values.
				boolean parametersAreGood = runAndRenderTask.validateParameters( );
				System.out.println("parametersAreGood: " + parametersAreGood);
				paramTask.close();
				
				// BIRT PDF Export
				DateFormat dateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
				Date date = new Date();
				PDFRenderOption pdfOptions = new PDFRenderOption();
				String outputFileName = getOutputDirectory() + "Output_" +  dateFormat.format(date) + PDF_FILE_EXTENSION;
				pdfOptions.setOutputFileName(outputFileName);
				pdfOptions.setOutputFormat("pdf");
				runAndRenderTask.setRenderOption(pdfOptions);
				runAndRenderTask.run();
				
				
				
//				Map appContext = task.getAppContext();
//				FileInputStream fileInputStream;
//				try {
//					fileInputStream = new FileInputStream("C:/Users/arendt.INFORMATIK/Desktop/test.xml");
//					appContext.put("org.eclipse.datatools.enablement.oda.xml.inputStream", fileInputStream);
//					//Use closeInputStream (true) to let BIRT automatically close the inputStream.
//					appContext.put("org.eclipse.datatools.enablement.oda.xml.closeInputStream", "true"); 
//				} catch (FileNotFoundException e) {
//					e.printStackTrace();
//				}
				
				
				
//				// HTML Export
//				HTMLRenderOption htmlOptions = new HTMLRenderOption();
//				DateFormat dateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
//				Date date = new Date();
//				String outputFileName = getDesignDirectory() + "Output_" +  dateFormat.format(date) + HTML_FILE_EXTENSION;
//				htmlOptions.setOutputFileName(outputFileName);
//				htmlOptions.setOutputFormat("html");
//				htmlOptions.setEmbeddable(false);
//				task.setRenderOption(htmlOptions);
//				task.run();
							
//				String documentFileName = getDesignDirectory() + "Metrics" + DOCUMENT_FILE_EXTENSION;
//				task.run(documentFileName);
				runAndRenderTask.close();
			} catch (EngineException e) {
				e.printStackTrace();
			}
			engine.destroy();
		}		
	}
	
	

	private class SaveAction extends Action {

		Shell shell;

		SaveAction(Shell shell) {
			super();
			this.shell = shell;
		}

		@SuppressWarnings("unchecked")
		public void run() {
			FileDialog fd = new FileDialog(shell, SWT.SAVE);
			fd.setText("Save Results");
			fd.setFilterPath("C:/");
			String[] filterExt = { "*.xml", "*.*" };
			fd.setFilterExtensions(filterExt);
			String selected = fd.open();
			System.out.println("Selected: " + selected);
			if (selected != null)
				XMLResultsFile.saveResults(selected, (List<Result>)viewer.getInput());
		}
	}
	
	private class ColumnSortListener implements Listener{
		private TableColumn column;
		public ColumnSortListener(TableColumn column) {
			this.column = column;
		}
		
		private void orderByColumn(TableColumn column) {
			ResultsViewContentProvider provider = (ResultsViewContentProvider)viewer.getContentProvider();
			if(column.getText().equals(METRIC_COLUMN_LABEL))
				provider.sortContent(ResultsViewContentProvider.METRIC_NAME);
			if(column.getText().equals(RESULT_COLUMN_LABEL))
				provider.sortContent(ResultsViewContentProvider.VALUE);
			if(column.getText().equals(DESCRIPTION_COLUMN_LABEL))
				provider.sortContent(ResultsViewContentProvider.DESCRIPTION);
			if(column.getText().equals(TIME_COLUMN_LABEL))
				provider.sortContent(ResultsViewContentProvider.TIME);
			if(column.getText().equals(CONTEXT_COLUMN_LABEL))
				provider.sortContent(ResultsViewContentProvider.CONTEXT);
		}



		@Override
		public void handleEvent(Event event) {
			orderByColumn(column);		
		}
		
		
	}
	
	public String getOutputDirectory() {
		String directory = "";
		final Bundle bundle = Activator.getDefault().getBundle();
		try {
			directory = FileLocator.toFileURL(bundle.getEntry(OUTPUT_DIR)).getFile();
		} catch (final IOException e) {
			e.printStackTrace();
		}
		return directory;
	}	

	public String getDesignDirectory() {
		String directory = "";
		final Bundle bundle = Activator.getDefault().getBundle();
		try {
			directory = FileLocator.toFileURL(bundle.getEntry(DESIGN_DIR)).getFile();
		} catch (final IOException e) {
			e.printStackTrace();
		}
		return directory;
	}	

}