package de.unimarburg.swt.emf.metrics.gui.wizard;

import java.io.File;
import org.eclipse.core.resources.IProject;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

import de.unimarburg.swt.emf.metrics.utils.FileUtils;

public class HenshinDataWizardPage extends WizardPage implements Listener {

	private static final String HENSHIN_EXT = ".henshin";
	private static final String PAGE_NAME = "org.eclipse.emf.metrics.wizards.HenshinDataWizardPage";
	private static final String TITLE = "Henshin Data";
	private static final String DESCRIPTION = "Specify Henshin transformation files for the Metric. \n"
			+ "(required fields are denoted by \"*\")";
	private Combo preCombo, transformationCombo, postCombo;
	private String pre, transf, post;
	private boolean initializing = false;

	@Override
	public void createControl(Composite parent) {
		final Shell shell = parent.getShell();
		Composite container = new Composite(parent, SWT.NULL);
		final GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		container.setLayout(layout);
		setControl(container);
		createTextFields(container, shell);
		this.setPageComplete(false);
	}

	@Override
	public boolean canFlipToNextPage() {
		return false;
	}

	@Override
	public void handleEvent(Event event) {
		updatePageComplete();
		getWizard().getContainer().updateButtons();
	}

	protected HenshinDataWizardPage() {
		super(PAGE_NAME);
		setTitle(TITLE);
		setDescription(DESCRIPTION);
	}

	private void createTextFields(Composite container, Shell shell) {
		Label label;
		Button button;
		GridData gridData;
		Group group;
		GridLayout layout;
		group = new Group(container, SWT.NONE);
		group.setText("Henshin files");
		layout = new GridLayout();
		layout.numColumns = 3;
		group.setLayout(layout);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		group.setLayoutData(gridData);
		// - Pre -
		label = new Label(group, SWT.NONE);
		label.setText("Pre-Trensformation file:");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		preCombo = new Combo(group, SWT.READ_ONLY);
		preCombo.addListener(SWT.Modify, this);
		preCombo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		button = new Button(group, SWT.NONE);
		button.addSelectionListener(new HenshinDataSelectionAdapter(shell,
				preCombo));
		button.setText("Import");
		// - Transf -
		label = new Label(group, SWT.NONE);
		label.setText("Metric-Calculation file *:");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		transformationCombo = new Combo(group, SWT.READ_ONLY);
		transformationCombo.addListener(SWT.Modify, this);
		transformationCombo
				.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		button = new Button(group, SWT.NONE);
		button.addSelectionListener(new HenshinDataSelectionAdapter(shell,
				transformationCombo));
		button.setText("Import");
		// - Post -
		label = new Label(group, SWT.NONE);
		label.setText("Post-Trensformation file:");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		postCombo = new Combo(group, SWT.READ_ONLY);
		postCombo.addListener(SWT.Modify, this);
		postCombo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		button = new Button(group, SWT.NONE);
		button.addSelectionListener(new HenshinDataSelectionAdapter(shell,
				postCombo));
		button.setText("Import");
	}

	protected void initContents() {
		IProject project = ((NewMetricWizard) getWizard()).getTargetProject();
		String path = project.getLocationURI().getPath() + NewMetricWizard.TRANSFORMATIONS_DIR;
		setCombos(FileUtils.getAllHenshinFiles(path));
	}

	private void setCombos(File[] files) {
		initializing = true;
		preCombo.removeAll();
		transformationCombo.removeAll();
		postCombo.removeAll();
		if (files != null) {
			for (File file : files) {
				preCombo.add(file.getName());
				transformationCombo.add(file.getName());
				postCombo.add(file.getName());
			}
		}
		initializing = false;
	}

	/**
	 * berprft die inhalte der Textfelder und erzeugt entschprechende
	 * Meldungen im Wizardfenster.
	 */
	private void updatePageComplete() {
		if (!initializing)
			if (!checkTransformationTextField()) {
				setMessage(
						"Metric-Calculation file not specified, or file does not exist.",
						ERROR);
				this.setPageComplete(false);
				getWizard().getContainer().updateButtons();
			} else {
				setMessage("");
				saveContents();
				this.setPageComplete(true);
				getWizard().getContainer().updateButtons();
			}
	}

	private boolean checkTransformationTextField() {
		final String transf = this.transformationCombo.getText();
		if (!transf.isEmpty())
			return true;
		return false;
	}

	private void saveContents() {
		this.pre = this.preCombo.getText();
		this.transf = this.transformationCombo.getText();
		this.post = this.postCombo.getText();
	}

	public String getPre() {
		return pre;
	}

	public String getTransf() {
		return transf;
	}

	public String getPost() {
		return post;
	}

	private void addToCombos(String name){
		preCombo.add(name,0);
		transformationCombo.add(name,0);
		postCombo.add(name,0);
	}
	

	// -------------------------------------------------------------------------------
	private class HenshinDataSelectionAdapter extends SelectionAdapter {

		private final Combo combo;
		private final Shell shell;

		public void widgetSelected(SelectionEvent e) {
			String result = browseForTransformation(shell);
			if (result != null)
				if (!result.isEmpty()) {
					result=result.replace('\\', '/');
					int i = result.lastIndexOf("/");
					String name = result.substring(i + 1);
					String destination = ((NewMetricWizard) getWizard()).getTargetProject().getLocationURI().getPath()+NewMetricWizard.TRANSFORMATIONS_DIR;
					FileUtils.copyFile(result, destination, name);
					//combo.add(name, 0);
					addToCombos(name);
					combo.select(0);
				}
		}

		private HenshinDataSelectionAdapter(Shell shell, Combo combo) {
			this.shell = shell;
			this.combo = combo;
		}

		private String browseForTransformation(Shell shell) {
			FileDialog fd = new FileDialog(shell, SWT.OPEN);
			fd.setText("Select Henshin-File");
			if (((NewMetricWizard) getWizard()).getTargetProject() != null)
				fd.setFilterPath(((NewMetricWizard) getWizard()).getTargetProject().getLocation().toString());
			String[] filterExt = { "*"+HENSHIN_EXT };
			fd.setFilterExtensions(filterExt);
			String selected = fd.open();
			return selected;
		}
	}

}
