package de.unimarburg.swt.emf.metrics.gui.wizard;

import java.util.Arrays;

import org.eclipse.core.resources.IProject;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;

public class MetricDataWizardPage extends WizardPage implements Listener{

	public static final String GENERATE_SKELETON_SOURCE_COMBO_ENTRY = "Generate Skeleton";
	public static final String HENSHIN_SOURCE_COMBO_ENTRY = "Henshin";
	public static final String COMPOSITE_SOURCE_COMBO_ENTRY = "Composite";
	public static final String OCL_SOURCE_COMBO_ENTRY = "OCL";
	private static final String PAGE_NAME = "org.eclipse.emf.metrics.wizards.MetricDataWizardPage";
	private static final String PAGE_TITLE = "Metric Data";
	private static final String PAGE_DESCRIPTION = "Specify Metric data. \n"
												+ "(required fields are denoted by \"*\")";
	private Text nameTextField, idTextField, descriptionTextField, contextTextField;
//	private Text valueTypeTextField;
//	private Text metamodelTextField;
	private Combo projectCombo, sourceCombo, metamodelCombo;
	private boolean initialization=false;
	

	
	@Override
	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		final GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		container.setLayout(layout);
		createTextFields(container);
		initContents();
		setControl(container);
		this.setPageComplete(false);
	}
	

	@Override
	public boolean canFlipToNextPage() {
		if(!initialization){
			if(this.sourceCombo.getText().equals(GENERATE_SKELETON_SOURCE_COMBO_ENTRY)){
				return false;
			}
		}
		return this.isPageComplete();
	}
	
	@Override
	public void handleEvent(Event event) {
		if (event.widget == sourceCombo)
			((NewMetricWizard)getWizard()).setSourceType(sourceCombo.getText());
		if (event.widget == projectCombo)
			((NewMetricWizard)getWizard()).setTargetProject(projectCombo.getText());
		if(!initialization){
			updatePageComplete();
			getWizard().getContainer().updateButtons();		
		}
	}
	
	public WizardPage getNextPage() {
		if (((NewMetricWizard)getWizard()).getSourceType().equals(HENSHIN_SOURCE_COMBO_ENTRY)) {
			HenshinDataWizardPage page = ((NewMetricWizard)getWizard()).getHenshinPage();
			((NewMetricWizard)getWizard()).getHenshinPage().initContents();
			return page;
		}
		if (((NewMetricWizard)getWizard()).getSourceType().equals(COMPOSITE_SOURCE_COMBO_ENTRY)) {
			CompositeDataWizardPage page = ((NewMetricWizard)getWizard()).getCompositePage();
			((NewMetricWizard)getWizard()).getCompositePage().fillTables();
			page.setPageComplete(false);
			return page;
		}
		if (((NewMetricWizard)getWizard()).getSourceType().equals(OCL_SOURCE_COMBO_ENTRY)) {
			OCLDataWizardPage page = ((NewMetricWizard)getWizard()).getOCLPage();
			page.setPageComplete(false);
			return page;
		}
		if (((NewMetricWizard)getWizard()).getSourceType().equals(GENERATE_SKELETON_SOURCE_COMBO_ENTRY)) {
			return null;
		}
		return null;
	}

	protected MetricDataWizardPage() {
		super(PAGE_NAME);
		setTitle(PAGE_TITLE);
		setDescription(PAGE_DESCRIPTION);
	}
	
	//TODO REFACTORING: split to smaller methods
	private void createTextFields(Composite container) {
		Label label;
		GridData gridData;
		Group group;
		GridLayout layout;
		// - Project -
		group = new Group(container, SWT.NONE);
		group.setText("Project");
	    layout = new GridLayout();
	    layout.numColumns = 2;
	    group.setLayout(layout);
	    gridData = new GridData(GridData.FILL_HORIZONTAL);
	    group.setLayoutData(gridData);
		// col:1
		label = new Label(group, SWT.NONE);
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		label.setText("Target project *:");
		// col:2
		projectCombo = new Combo(group, SWT.READ_ONLY);
		final GridData projectComboData = new GridData(GridData.FILL_HORIZONTAL);
		projectCombo.setLayoutData(projectComboData);
		//projectCombo.setText("Select Project");
		projectCombo.addListener(SWT.Selection, this);
		// - Metric Data -
		group = new Group(container, SWT.NONE);
		group.setText("Metric data");
	    layout = new GridLayout();
	    layout.numColumns = 2;
	    group.setLayout(layout);
	    gridData = new GridData(GridData.FILL_HORIZONTAL);
	    group.setLayoutData(gridData);
		// - Name -
		// col:1
		label = new Label(group, SWT.NONE);
		label.setText("Name *:");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		// col:2
		nameTextField = new Text(group, SWT.BORDER);
		nameTextField.addListener(SWT.Modify, this);
		nameTextField.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		// - Id -
		// col:1
		label = new Label(group, SWT.NONE);
		label.setText("Id *:");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		// col:2
		idTextField = new Text(group, SWT.BORDER);
		idTextField.addListener(SWT.Modify, this);
		idTextField.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		// - Description -
		// col:1
		label = new Label(group, SWT.NONE);
		label.setText("Description:");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		// col:2
		descriptionTextField = new Text(group, SWT.BORDER);
		descriptionTextField.addListener(SWT.Modify, this);
		descriptionTextField.setLayoutData(new GridData(
				GridData.FILL_HORIZONTAL));
		// - Metamodel -
		// col:1
		label = new Label(group, SWT.NONE);
		label.setText("Metamodel *:");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		// col:2
		metamodelCombo = new Combo(group, SWT.BORDER);
		metamodelCombo.addListener(SWT.Selection, this);
		metamodelCombo.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
//		metamodelTextField = new Text(group, SWT.BORDER);
//		metamodelTextField.addListener(SWT.Modify, this);
//		metamodelTextField
//				.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		// - Context -
		// col:1
		label = new Label(group, SWT.NONE);
		label.setText("Context *:");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		// col:2
		contextTextField = new Text(group, SWT.BORDER);
		contextTextField.addListener(SWT.Modify, this);
		contextTextField.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		// - ValueType -
		// col:1
//		label = new Label(group, SWT.NONE);
//		label.setText("Value type:");
//		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
//		// col:2
//		valueTypeTextField = new Text(group, SWT.BORDER );
//		valueTypeTextField.addListener(SWT.Modify, this);
//		valueTypeTextField.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		// - Source -
		group = new Group(container, SWT.NONE);
		group.setText("Source");
	    layout = new GridLayout();
	    layout.numColumns = 2;
	    group.setLayout(layout);
	    gridData = new GridData(GridData.FILL_HORIZONTAL);
	    group.setLayoutData(gridData);
		// col:1
		label = new Label(group, SWT.NONE);
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		label.setText("Source *:");
		// col:2
		sourceCombo = new Combo(group, SWT.READ_ONLY);
		final GridData comboData = new GridData(GridData.FILL_HORIZONTAL);
		sourceCombo.setLayoutData(comboData);
		sourceCombo.setItems(new String[] { GENERATE_SKELETON_SOURCE_COMBO_ENTRY, HENSHIN_SOURCE_COMBO_ENTRY, COMPOSITE_SOURCE_COMBO_ENTRY, OCL_SOURCE_COMBO_ENTRY });
		sourceCombo.setText("Select metric source type");
		sourceCombo.addListener(SWT.Selection, this);
	}
	
	private void initContents() {
		initialization=true;
		initProjectsAndMetamodels();
		initFromSelectedEObject();
		initialization=false;
	}
	
	private void initProjectsAndMetamodels(){
		for (IProject project : ((NewMetricWizard)getWizard()).getProjects()) {
			projectCombo.add(project.getName());
		}
		Object [] metamodelObjects = EPackage.Registry.INSTANCE.keySet().toArray(new Object[EPackage.Registry.INSTANCE.size()]);
        Arrays.sort(metamodelObjects);
		for(Object object : metamodelObjects){
			metamodelCombo.add(object.toString());
		}
	}
	
	private void initFromSelectedEObject(){
		EObject selectedEObject = ((NewMetricWizard)getWizard()).getSelectedEObject();;
		if (selectedEObject != null) {
			String metamodel = ((EPackage) selectedEObject.eClass().eContainer()).getNsURI();
			String context = selectedEObject.eClass().getInstanceClassName();
			for(int i = 0; i < metamodelCombo.getItems().length; i++){
				if(metamodelCombo.getItem(i).equals(metamodel))
					metamodelCombo.select(i);
			}
			this.contextTextField.setText(context);
		}
	}

	/**
	 * Wird jedes mal ausgefhr wenn sich der Inhalt eines Textfeldes im Wizard
	 * verndert. berprft die inhalte der Textfelder und erzeugt
	 * entschprechende Meldungen im Wizardfenster.
	 */
	private void updatePageComplete() {
		if(nameTextField.getText().isEmpty()){
			this.setMessage("Metric name is not specified.", ERROR);
			this.setPageComplete(false);
		}else
		if(idTextField.getText().isEmpty()){
			this.setMessage("Metric ID is not specified.", ERROR);
			this.setPageComplete(false);
		}else
		if(metamodelCombo.getText().isEmpty()){
			this.setMessage("Metric metamodel is not specified.", ERROR);
			this.setPageComplete(false);
		}else
		if( contextTextField.getText().isEmpty()){
			this.setMessage("Metric context is not specified.", ERROR);
			this.setPageComplete(false);
		}else
		if(!checkProject()){
			this.setMessage("Target project for the Metric is not specified.", ERROR);
			this.setPageComplete(false);
		}else
		if(!checkSourceType()){
			this.setMessage("Source type for the Metric is not specified.", ERROR);
			this.setPageComplete(false);
		}else{
			this.setMessage("");
			this.setPageComplete(true);
			saveTextFieldValues();
		}
	}
	
	private boolean checkProject(){
		if(!projectCombo.getText().isEmpty())
			return true;
		return false;
	}
	
	private boolean checkSourceType(){
		if(sourceCombo.getText().equals(OCL_SOURCE_COMBO_ENTRY) || sourceCombo.getText().equals(HENSHIN_SOURCE_COMBO_ENTRY) || sourceCombo.getText().equals(COMPOSITE_SOURCE_COMBO_ENTRY) || sourceCombo.getText().equals(GENERATE_SKELETON_SOURCE_COMBO_ENTRY))
			return true;
		return false;
	}
	
	private void saveTextFieldValues(){
		((NewMetricWizard)getWizard()).setName(this.nameTextField.getText());
		((NewMetricWizard)getWizard()).setId(this.idTextField.getText());
		((NewMetricWizard)getWizard()).setDescription(this.descriptionTextField.getText());
		((NewMetricWizard)getWizard()).setMetamodel(this.metamodelCombo.getText());
		((NewMetricWizard)getWizard()).setContext(this.contextTextField.getText());
		((NewMetricWizard)getWizard()).setValueType("");
	}
			
//TODO
//	if (! XmlUtil.pluginXmlExists(projectName)){
//			XmlUtil.createPluginXml(projectName);
//		}
//		List<RefactoringConfig> refactoringConfig = XmlUtil.getRefactoringConfig(projectName);
//		boolean exists = false;
//		for (RefactoringConfig refCon : refactoringConfig){
//			if (refCon.getRefactoringId().equals(id)){
//				exists = true;
//				break;
//			}
//		}
//		if (exists){
//			setMessage("There is already a refactoring with id '" + id + "' in project '" + projectName + "'!");
//			setPageComplete(false);

//	private LinkedList<String> getAllContextsForMetamodel(String metamodel) {
//	LinkedList<String> results = new LinkedList<String>();
//	for (Metric metric : EMFMetrics.getAllMetrics()) {
//		if (metric.getMetamodel().equals(metamodel)) {
//			String context = metric.getContext();
//			boolean found = false;
//			for (String result : results) {
//				if (result.equals(context))
//					found = true;
//			}
//			if (!found)
//				results.add(context);
//		}
//	}
//	return results;
//}

//private LinkedList<String> getAllMetamodels() {
//	LinkedList<String> results = new LinkedList<String>();
//	for (Metric metric : EMFMetrics.getAllMetrics()) {
//		String metamodel = metric.getMetamodel();
//		boolean found = false;
//		for (String result : results) {
//			if (result.equals(metamodel))
//				found = true;
//		}
//		if (!found)
//			results.add(metamodel);
//	}
//	return results;
//}

}
	

