package de.unimarburg.swt.emf.metrics.gui.wizard;

import java.lang.reflect.InvocationTargetException;
import java.util.LinkedList;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectNature;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;

import de.unimarburg.swt.emf.metrics.EMFMetrics;
import de.unimarburg.swt.emf.metrics.model.Metric;
import de.unimarburg.swt.emf.metrics.model.generator.CompositeMetricInfo;
import de.unimarburg.swt.emf.metrics.model.generator.HenshinMetricInfo;
import de.unimarburg.swt.emf.metrics.model.generator.MetricInfo;
import de.unimarburg.swt.emf.metrics.model.generator.OCLMetricInfo;
import de.unimarburg.swt.emf.metrics.operations.IOperation;
import de.unimarburg.swt.emf.metrics.utils.SelectionUtils;

public class NewMetricWizard extends Wizard implements INewWizard {

	protected static final String TRANSFORMATIONS_DIR = "/transformations/";
	private final String WINDOW_TITLE = "New Metric";
	private IStructuredSelection initialSelection;
	private MetricDataWizardPage dataPage;
	private HenshinDataWizardPage henshinPage;
	private CompositeDataWizardPage compositePage;
	private OCLDataWizardPage oclPage;
	private String name, id, description, metamodel, context, valueType, oclExpression;
	private String sourceType;
	private LinkedList<IProject> projects;
	private IProject targetProject;
		
	public NewMetricWizard() {
		
	}
		
	@Override
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		initialSelection = selection;	
		initProjects();
	}
	
	@Override
	public void addPages(){
		setWindowTitle(WINDOW_TITLE);
		dataPage = new MetricDataWizardPage();
		addPage(dataPage);
		henshinPage = new HenshinDataWizardPage();
		addPage(henshinPage);
		compositePage = new CompositeDataWizardPage();
		addPage(compositePage);
		oclPage = new OCLDataWizardPage();
		addPage(oclPage);
	}
	
	@Override
	public boolean canFinish(){
		if( dataPage.isPageComplete() && this.sourceType.equals(MetricDataWizardPage.GENERATE_SKELETON_SOURCE_COMBO_ENTRY))
			return true;
		if(dataPage.isPageComplete() && (henshinPage.isPageComplete() || compositePage.isPageComplete() || oclPage.isPageComplete()))
			return true;
		return false;
	}
	
	@Override
	public boolean performFinish() {
		try{
			getContainer().run(true, true, new IRunnableWithProgress(){
				public void run(IProgressMonitor monitor)throws InvocationTargetException, InterruptedException{
					EMFMetrics.createNewMetric(monitor, getMetricInfo(), targetProject);
				}
			});
		}
		catch(InvocationTargetException e){
			e.printStackTrace();
			return false;
		}
		catch(InterruptedException e){
			e.printStackTrace();
			return false;
		}
		return true;
	}
	
	private void initProjects(){
		this.projects = new LinkedList<IProject>();
		IProject[] allProjects = ResourcesPlugin.getWorkspace().getRoot().getProjects();
		for (IProject project : allProjects) {
			if (project.isOpen()) {
				IProjectNature nature = null;
				try {
					nature = project.getNature("org.eclipse.pde.PluginNature");
				} catch (CoreException e) {
					e.printStackTrace();
				}
				if (null != nature) 
					this.projects.add(project);
			}
		}
	}
	
	private MetricInfo getMetricInfo(){
		String proj = this.targetProject.getLocation().toString();
		System.out.println("target project for new metric:"+proj);
		// - Henshin -
		MetricInfo info = null;
		if(sourceType.equals(MetricDataWizardPage.HENSHIN_SOURCE_COMBO_ENTRY)){
			String pre = targetProject.getLocationURI().getPath()+TRANSFORMATIONS_DIR+henshinPage.getPre();
			String transf = targetProject.getLocationURI().getPath()+TRANSFORMATIONS_DIR+henshinPage.getTransf();
			String post = targetProject.getLocationURI().getPath()+TRANSFORMATIONS_DIR+henshinPage.getPost();
		    info = new HenshinMetricInfo(this.name, this.id, this.description, this.metamodel, this.context, this.valueType, proj, pre, transf, post);
		}
		// - Composite -
		if(sourceType.equals(MetricDataWizardPage.COMPOSITE_SOURCE_COMBO_ENTRY)){
			Metric first = compositePage.getFirstMetric();
			Metric second = compositePage.getSecondMetric();
			String operationName = compositePage.getOperationName();
			IOperation operation = EMFMetrics.getOperation(operationName);
			info = new CompositeMetricInfo(this.name, this.id, this.description, this.metamodel, this.context, this.valueType, proj, first, second, operation);
		}
		// - OCL -
		if(sourceType.equals(MetricDataWizardPage.OCL_SOURCE_COMBO_ENTRY)){
			info = new OCLMetricInfo(this.name, this.id, this.description, this.metamodel, this.context, this.valueType, proj, oclExpression);
		}
		// - Skeleton -
		if(sourceType.equals(MetricDataWizardPage.GENERATE_SKELETON_SOURCE_COMBO_ENTRY)){
		    info = new MetricInfo(this.name, this.id, this.description, this.metamodel, this.context, this.valueType, proj);
		}
		return info;
	}
	
	
	public HenshinDataWizardPage getHenshinPage() {
		return henshinPage;
	}

	public CompositeDataWizardPage getCompositePage() {
		return compositePage;
	}
	
	public MetricDataWizardPage getDataPage() {
		return dataPage;
	}
	
	protected EObject getSelectedEObject(){
		return SelectionUtils.getSelectedEObject(initialSelection);
	}
	
	public LinkedList<IProject> getProjects(){
		return projects;
	}
	
	public IProject getTargetProject(){
		return targetProject;
	}
	
	public String getSourceType(){
		return sourceType;
	}
	
	public String getMetamodel() {
		return metamodel;
	}

	public String getContext() {
		return context;
	}
	
	public void setName(String name) {
		this.name = name;
	}

	public void setId(String id) {
		this.id = id;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public void setMetamodel(String metamodel) {
		this.metamodel = metamodel;
	}

	public void setContext(String context) {
		this.context = context;
	}

	public void setValueType(String valueType) {
		this.valueType = valueType;
	}

	public void setTargetProject(String projectName){
		for(IProject project : projects)
			if(project.getName().equals(projectName))
		this.targetProject=project;
	}
	public void setSourceType(String sourceType){
		this.sourceType = sourceType;
	}

	public OCLDataWizardPage getOCLPage() {
		return oclPage;
	}
	
	public void setOCLExpression(String expression){
		this.oclExpression = expression;
	}
	
}
