package de.unimarburg.swt.emf.metrics.interfaces;


import de.unimarburg.swt.emf.metrics.interfaces.ICalculateClass;

import de.unimarburg.swt.emf.metrics.model.HenshinTransformationsPack;
import de.unimarburg.swt.emf.metrics.operations.IOperation;

/**
 * This interface defines the Methods needed to generate a new metric
 * using other already defined metrics. 
 * The methods provide informations needed by the new metric.
 * 
 * @author Pawel Stepien
 */
public interface ICompositeCalculateClass extends ICalculateClass{
		
	/**
	 * Returns a String representation of the calculate operation used in this class.
	 * 
	 * @return operation Class
	 */
	abstract public String getCalculateOperation();
	
	/**
	 * Returns the <i>HenshinTransformationsPack</i> elements used in this class.
	 * 
	 * @return operations
	 */
	abstract public HenshinTransformationsPack[] getTransformationPacks();
	
	/**
	 * Returns the <i>IOperation</i> implementing operations used in this class.
	 * 
	 * @return operations
	 */
	abstract public IOperation[] getOperations();
}
