package de.unimarburg.swt.emf.metrics.model;

/**
 * This class provides a container for the paths to henshin-transformation files 
 * used in a <i>ICalculateClass</i>. 
 * 
 * @author Pawel Stepien
 */
public class HenshinTransformationsPack {

	private String prePath, tranfPath, postPath;

	/**
	 * The constructor that creates a new <i>EMFMetrics</i> controller class.
	 * 
	 * @param path to the pre-transformation file
	 * @param path to the count-transformation file
	 * @param path to the post-transformation file
	 */
	public HenshinTransformationsPack(String prePath, String tranfPath,
			String postPath) {
		this.prePath = prePath;
		this.tranfPath = tranfPath;
		this.postPath = postPath;
	}

	/**
	 * Returns the path to the pre-transformation file.
	 * 
	 * @return pre-transformation path
	 */
	public String getPrePath() {
		return prePath;
	}

	/**
	 * Returns the path to the count-transformation file.
	 * 
	 * @return count-transformation path
	 */
	public String getTransformationPath() {
		return tranfPath;
	}

	/**
	 * Returns the path to the post-transformation file.
	 * 
	 * @return post-transformation path
	 */
	public String getPostPath() {
		return postPath;
	}

	/**
	 * Returns the name of the pre-transformation file.
	 * 
	 * @return pre-transformation file name
	 */
	public String getPrePathFileName() {
		int index = prePath.lastIndexOf("/");
		String name = prePath.substring(index+1);
		return name;
	}

	/**
	 * Returns the name of the count-transformation file.
	 * 
	 * @return count-transformation file name
	 */
	public String getTransformationPathFileName() {
		int index = tranfPath.lastIndexOf("/");
		String name = tranfPath.substring(index+1);
		return name;
	}

	/**
	 * Returns the name of the post-transformation file.
	 * 
	 * @return post-transformation file name
	 */
	public String getPostPathFileName() {
		int index = postPath.lastIndexOf("/");
		String name = postPath.substring(index+1);
		return name;
	}
	
}
