package de.unimarburg.swt.emf.metrics.model.generator;


import de.unimarburg.swt.emf.metrics.model.generator.MetricInfo;

import de.unimarburg.swt.emf.metrics.interfaces.ICompositeCalculateClass;
import de.unimarburg.swt.emf.metrics.model.HenshinTransformationsPack;
import de.unimarburg.swt.emf.metrics.model.Metric;
import de.unimarburg.swt.emf.metrics.operations.IOperation;
import de.unimarburg.swt.emf.metrics.operations.Operations;


/**
 * This class provides a container for the information needed to generate a new metric
 * using two already existing metrics.
 * This class extends the abstract <i>MetricInfo</i> class providing additional functionality. 
 * 
 * @author Pawel Stepien
 */
public class CompositeMetricInfo extends MetricInfo  {

	private Metric firstMetric, secondMetric;
	private IOperation operation;
	
	/**
	 * The constructor that creates a new <i>CompositeMetricInfo</i> object.
	 * 
	 * @param name
	 * @param id
	 * @param description
	 * @param metamodel
	 * @param context
	 * @param valueType
	 * @param targetProject
	 * @param firstMetric
	 * @param secondMetric
	 * @param operation
	 */
	public CompositeMetricInfo(String name, String id, String description, String metamodel, String context, String valueType, String project, 
								Metric firstMetric, Metric secondMetric, IOperation operation) {
		super(name, id, description, metamodel, context, valueType, project);
		this.firstMetric = firstMetric;
		this.secondMetric = secondMetric;
		this.operation = operation;
	}

	/**
	 * Returns the first metric for the composition.
	 * 
	 * @return first metric
	 */
	public Metric getFirstMetric() {
		return firstMetric;
	}

	/**
	 * Returns the second metric for the composition.
	 * 
	 * @return second metric
	 */
	public Metric getSecondMetric() {
		return secondMetric;
	}

	/**
	 * Returns the operation for the metrics composition.
	 * 
	 * @return operation
	 */
	public IOperation getOperation() {
		return operation;
	}
	
	
	/**
	 * Returns the operationname for the metrics composition.
	 * 
	 * @return operationname
	 */
	public String getOperationName(){
		return Operations.getOperationName(operation);
	}
	
//	/**
//	 * Returns a <i>String</i> representation of the <i>HenshinTransformationPack</i> 
//	 * objects used in in the new metric.
//	 * 
//	 * @return transformationPacks 
//	 */
//	public String getTransformationsPacksString(){
//		return getTransformationsPacks(secondMetric)+", "+getTransformationsPacks(firstMetric);
//	}
	
//	/**
//	 * Returns a <i>String</i> representation of the <i>IOperation</i> 
//	 * objects used in the new metric.
//	 * 
//	 * @return operations 
//	 */
//	public String getOperationsString(){
//		final String first = getOperationsString(firstMetric);
//		final String second = getOperationsString(secondMetric);
//		final String operation = getOperationString();
//		if(first!=null)
//			if(second!=null)
//				return 	second + ", " + first + ", " + operation;
//			else
//				return first + ", " + operation;
//		else
//			if(second!=null)
//				return 	second + ", " + operation ;
//		return operation;
//	}
	
//	/**
//	 * Returns a <i>String</i> representation of the calculate operation for the new metric.
//	 * 
//	 * @return transformationPacks 
//	 */
//	public String getCalculateOperationString(){
//		return "result(opStack.pop(), "+((ICompositeCalculateClass)firstMetric.getCalculateClass()).getCalculateOperation()+", "+((ICompositeCalculateClass)secondMetric.getCalculateClass()).getCalculateOperation()+")";
//	}
//	
	
//	private String getOperationString(){
//		return  "new "+operation.getClass().getSimpleName()+"()";
//	}
	
//	private String getOperationsString(Metric metric){
//		String result = "";
//		IOperation[] operations = ((ICompositeCalculateClass)metric.getCalculateClass()).getOperations();
//		if(operations==null)
//			return null;
//		String tempResult;
//		for(int i=0; i<operations.length; i++){
//			tempResult = "new "+operations[i].getClass().getSimpleName()+"()";
//			if(i==0)
//				result = tempResult;
//			else	
//				result = result + "," + tempResult;
//		}
//		return result;
//	}
//
//	private String getTransformationsPacks(Metric metric){
//		String result = "";
//		HenshinTransformationsPack[] packs = ((ICompositeCalculateClass)metric.getCalculateClass()).getTransformationPacks();
//		String tempResult;
//		for(int i=0; i<packs.length; i++){
//			String first;
//			if(packs[i].getPrePath().isEmpty())
//				first = "\"\"";
//			else
//				first = "getFullPath(\"transformations/"+packs[i].getPrePathFileName()+"\")";
//			String second = "getFullPath(\"transformations/"+packs[i].getTransformationPathFileName()+"\")";
//			String third;
//			if(packs[i].getPostPath().isEmpty())
//				third  = "\"\"";
//			else
//				third = "getFullPath(\"transformations/"+packs[i].getPostPathFileName()+"\")";
//			tempResult = "new HenshinTransformationsPack("+first +", "+second +", "+third +")";
//			if(i==0)
//				result = tempResult;
//			else	
//				result = result + "," + tempResult;
//		}
//		return result;
//	}
	
}
