package de.unimarburg.swt.emf.metrics.model.generator;

import de.unimarburg.swt.emf.metrics.model.generator.MetricInfo;

/**
 * This class provides a container for the information needed to generate a new metric
 * using henshin-transformations. 
 * This class extends the abstract <i>MetricInfo</i> class providing additional functionality. 
 * 
 * @author Pawel Stepien
 */
public class HenshinMetricInfo extends MetricInfo {

	private String pre, transf, post;
	
	/**
	 * The constructor that creates a new <i>HenshinMetricInfo</i> object.
	 * 
	 * @param name
	 * @param id
	 * @param description
	 * @param metamodel
	 * @param context
	 * @param valueType
	 * @param targetProject
	 * @param pre-transformation path
	 * @param count-transformation path
	 * @param post-transformation path
	 */
	public HenshinMetricInfo(String name, String id, String description,
			 String metamodel, String context, String valueType, String project, String pre, String transf, String post) {
		super(name, id, description, metamodel, context, valueType, project);
		this.pre=pre;
		this.transf=transf;
		this.post=post;
	}

	/**
	 * Returns the path to the pre-transformation file..
	 * 
	 * @return pre-transformation file path
	 */
	public String getPreTransformationPath() {
		return pre;
	}

	/**
	 * Returns the path to the count-transformation file..
	 * 
	 * @return count-transformation file path
	 */
	public String getTransformationPath() {
		return transf;
	}

	/**
	 * Returns the path to the post-transformation file..
	 * 
	 * @return post-transformation file path
	 */
	public String getPostTransformationPath() {
		return post;
	}
	
	/**
	 * Returns the name of the pre-transformation file.
	 * 
	 * @return pre-transformation file name
	 */
	public String getPreTransformationName() {
		int index = pre.lastIndexOf("/");
		String result = pre.substring(index+1);
		return result;
	}	
	
	/**
	 * Returns the name of the count-transformation file.
	 * 
	 * @return count-transformation file name
	 */
	public String getTransformationName() {
		int index = transf.lastIndexOf("/");
		String result = transf.substring(index+1);
		return result;
	}	
	
	/**
	 * Returns the name of the post-transformation file.
	 * 
	 * @return post-transformation file name
	 */
	public String getPostTransformationName() {
		int index = post.lastIndexOf("/");
		String result = post.substring(index+1);
		return result;
	}
	
	
}
