package de.unimarburg.swt.emf.metrics.utils;

import java.util.LinkedList;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.Platform;

import de.unimarburg.swt.emf.metrics.config.ExtensionPoint;
import de.unimarburg.swt.emf.metrics.interfaces.ICalculateClass;
import de.unimarburg.swt.emf.metrics.model.Metric;

public class MetricLoader {


	public static LinkedList<Metric> loadMetrics() {
		LinkedList<Metric> metrics = new LinkedList<Metric>();
		IConfigurationElement[] rawMetrics = Platform.getExtensionRegistry()
				.getConfigurationElementsFor(ExtensionPoint.EXTENSION_POINT_NAME);
		for (IConfigurationElement element : rawMetrics) {
			try {
				if (element.getName().equals(ExtensionPoint.METRIC_TAG)) {
					Metric metric = createMetric(element);
					if (null != metric) {
						metrics.add(metric);
					}
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return metrics;
	}

	private static Metric createMetric(IConfigurationElement rawMetric) {
		try {
			final String name = rawMetric.getAttribute(ExtensionPoint.METRIC_NAME_TAG);
			final String id = rawMetric.getAttribute(ExtensionPoint.METRIC_ID_TAG);
			final String description = rawMetric
					.getAttribute(ExtensionPoint.METRIC_DESCRIPTION_TAG);
			final String metamodel = rawMetric
					.getAttribute(ExtensionPoint.METRIC_METAMODEL_TAG);
			final String context = rawMetric.getAttribute(ExtensionPoint.METRIC_CONTEXT_TAG);
			final String valueType = rawMetric
					.getAttribute(ExtensionPoint.METRIC_VALUE_TYPE_TAG);
			final ICalculateClass calculateClass = (ICalculateClass) rawMetric
					.createExecutableExtension(ExtensionPoint.METRIC_CALCULATE_CLASS_TAG);
			return new Metric(name, description, metamodel, context, valueType,
					calculateClass, id);
		} catch (Throwable e) {
			e.printStackTrace();
			return null;
		}
	}

}
