package de.unimarburg.swt.emf.metrics.utils.xml;

import java.lang.reflect.InvocationTargetException;
import java.text.DecimalFormat;
import java.util.LinkedList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EOperation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import de.unimarburg.swt.emf.metrics.model.calculator.Result;

public class XMLResultsFile extends XMLFile{

	private static final String RESULTS_TAG = "results";
	
	public static void saveResults(String path, List<Result> results){
		final DocumentBuilder builder = createDocumentBuilder();
		if (builder != null) {
			final Document doc = builder.newDocument();		
			final Element root = doc.createElement(RESULTS_TAG);
			doc.appendChild(root);	
			createAllContextElements(root, doc, results);
			final Transformer transformer = createTransformer();
			final DOMSource source = new DOMSource(doc);		
			final StreamResult result = new StreamResult(path);			
			try {
				transformer.transform(source, result);
			} catch (final TransformerException e) {
				e.printStackTrace();
			}		
		}
	}
	
	private static void createAllContextElements(Element root, Document doc, List<Result> results) {
		List<List<EObject>> allContexts = new LinkedList<List<EObject>>();
		for(Result result : results)
			if(!allContexts.contains(result.getContext()))
				allContexts.add(result.getContext());
		for(List<EObject> context : allContexts)
			root.appendChild(createContextElement(doc, context, filterResults(results, context)));
	}
	
	private static Element createContextElement(Document doc, List<EObject> cont, List<Result> results){
		final Element context = doc.createElement("context");
		final Element contextType = doc.createElement("contextType");
		contextType.setTextContent(cont.get(0).eClass().getName());
		context.appendChild(contextType);
		final Element contextValue = doc.createElement("contextValue");
		contextValue.setTextContent(getContextValue(cont.get(0)));
		context.appendChild(contextValue);
		for (Object rawResult : results){
			Result result = (Result) rawResult;
			Element resultEntry = doc.createElement("metricResult");
			Element dateStamp = doc.createElement("date");
			dateStamp.setTextContent(result.getTimeStamp());
			resultEntry.appendChild(dateStamp);			
			Element metricName = doc.createElement("metricName");
			metricName.setTextContent(result.getMetric().getName());
			resultEntry.appendChild(metricName);			
			Element metricDescription = doc.createElement("metricDescription");
			metricDescription.setTextContent(result.getMetric().getDescription());
			resultEntry.appendChild(metricDescription);			
			Element resultValue = doc.createElement("resultValue");
			DecimalFormat df = new DecimalFormat( "0.00" );
			String value = df.format(result.getResultValue());
			resultValue.setTextContent(value);
			resultEntry.appendChild(resultValue);
			context.appendChild(resultEntry);
		}
		return context;
	}
	
//	private static Element createContextElement(Document doc, List<EObject> cont, List<Result> results){
//		final Element context = doc.createElement("context");
//		final Element contextValue = doc.createElement("contextValue");
//		contextValue.setTextContent(cont.get(0).toString());
//		context.appendChild(contextValue);
//		for (Object rawResult : results){
//			Result result = (Result)rawResult;
//			Element resultEntry = doc.createElement("metricResult");
//			Element dateStamp = doc.createElement("date");
//			dateStamp.setTextContent(result.getTimeStamp());
//			resultEntry.appendChild(dateStamp);
//			Element metricEntry = doc.createElement(METRIC_TAG);
//			Element metricName = doc.createElement(METRIC_NAME_TAG);
//			metricName.setTextContent(result.getMetric().getName());
//			metricEntry.appendChild(metricName);
//			Element metricDescription = doc.createElement("metricDescription");
//			metricDescription.setTextContent(result.getMetric().getDescription());
//			metricEntry.appendChild(metricDescription);
//			resultEntry.appendChild(metricEntry);
//			Element resultValue = doc.createElement("resultValue");
//			resultValue.setTextContent(""+result.getResultValue());
//			resultEntry.appendChild(resultValue);
//			context.appendChild(resultEntry);
//		}
//		return context;
//	}
	
	private static String getContextValue(EObject eObject) {
		for (EAttribute attribute : eObject.eClass().getEAllAttributes()) {
			if (attribute.getName().equals("name")) {
				return (String) eObject.eGet(attribute);
			}
		}
		
		String ret = "";
		ret = getEOperationValue(eObject, "getQualifiedName");
		if (! ret.isEmpty()) return ret;
		ret = getEOperationValue(eObject, "getName");
		if (! ret.isEmpty()) return ret;
		ret = getEOperationValue(eObject, "getLabel");
		if (! ret.isEmpty()) return ret;
		ret = getEOperationValue(eObject, "getID");
		if (! ret.isEmpty()) return ret;
		ret = getEOperationValue(eObject, "getId");
		if (! ret.isEmpty()) return ret;
		return eObject.toString();
	}

	private static String getEOperationValue(EObject eObject, String eOperationName) {
		for (EOperation eOperation : eObject.eClass().getEAllOperations()) {
			if (eOperation.getName().equals(eOperationName)) {
				try {
					return (String)eObject.eInvoke(eOperation, null);
				} catch (InvocationTargetException e) {
					e.printStackTrace();
					return "";
				}
			}
		}
		return "";
	}

	private static List<Result> filterResults(List<Result> results, List<EObject> context){
		List<Result> filteredResults = new LinkedList<Result>();
		for(Result result : results)
			if(result.getContext().equals(context))
				filteredResults.add(result);
		return filteredResults;
	}
	
}
