package de.unimarburg.swt.emf.modelsmell;

import java.util.Date;
import java.util.LinkedList;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TreeViewer;

import de.unimarburg.swt.emf.modelsmell.abstractclasses.MetricBasedModelSmellFinderClass;
import de.unimarburg.swt.emf.modelsmell.generator.ModelSmellGenerator;
import de.unimarburg.swt.emf.modelsmell.model.ModelSmell;
import de.unimarburg.swt.emf.modelsmell.model.generator.ModelSmellInfo;
import de.unimarburg.swt.emf.modelsmell.model.smellfinder.Configuration;
import de.unimarburg.swt.emf.modelsmell.model.smellfinder.Result;
import de.unimarburg.swt.emf.modelsmell.model.smellfinder.ResultViewEntry;
import de.unimarburg.swt.emf.modelsmell.model.smellfinder.treeviewer.ResultModel;
import de.unimarburg.swt.emf.modelsmell.smellfinder.ModelSmellFinder;
import de.unimarburg.swt.emf.modelsmell.util.DependenciesFile;
import de.unimarburg.swt.emf.modelsmell.util.MetricData;
import de.unimarburg.swt.emf.modelsmell.util.MetricStubLoader;
import de.unimarburg.swt.emf.modelsmell.util.ModelSmellLoader;
import de.unimarburg.swt.emf.modelsmell.util.xml.XMLPluginFile;
import de.unimarburg.swt.emf.modelsmell.util.xml.XMLProjectFile;
import de.unimarburg.swt.emf.modelsmell.views.ResultModelTreeViewer;

/**
 * Central controller class of EMF Smells. The logic for all the use cases is represented in one
 * <br>- or more methods of this class.
 * 
 * @author Matthias Burhenne
 *
 */

public class EMFModelSmells {
	
	private static LinkedList<ModelSmell> allSmells=null;
	private static LinkedList<Configuration> configurations;
	private static ModelSmellGenerator generator;
	private static LinkedList<ResultModel> resultModels = new LinkedList<ResultModel>();
//	private static TreeViewer resultModelViewer;
	private static ResultModelTreeViewer resultModelViewer;
	private static TableViewer resultsViewer = null;
	
	
	private static LinkedList<ResultViewEntry> resultsViewInput;
	
//	private XMLSmellGroupsConfigFile groupConfigFile;
	
	public EMFModelSmells(){
		resultsViewInput = new LinkedList<ResultViewEntry>();
		generator = ModelSmellGenerator.getGenerator();
		configurations = new LinkedList<Configuration>();
//		groupConfigFile = new XMLSmellGroupsConfigFile(instance.getClass().getProtectionDomain().getCodeSource().getLocation().getPath());
//		groupConfigFile.writeSmellGroupToConfigFile(new SmellGroup("name", "id", getAllModelSmells()));
	}
	
	public static void clearResultModels(){
		resultModels = new LinkedList<ResultModel>();
		resultModelViewer.setInput(resultModels);
	}
	
	/**
	 * Starts the generation of a new smell plugin in the target <i>IProject</i> based on 
	 * <br>- the provided <i>ModelSmellInfo</i>.
	 * <br>- Generates the finder class and updates the projects dependencies.
	 * 
	 * @param monitor - the monitor provided by the calling method 
	 * @param info - the ModelSmellInfo for the <i>ModelSmellGenerator</i>
	 * @param project - the target project
	 */
	public static void createNewModelSmell(IProgressMonitor monitor, ModelSmellInfo info, IProject project){
		generator.setModelSmellInfo(info);
		generator.createFinderClass(monitor);
		DependenciesFile.updateDependencies(info.getProjectName(), info);
		//String projectPath = info.getProject().getLocationURI().toString();
		String projectPath = info.getProject().getLocation().toString();
		XMLPluginFile.createModelSmellEntry(projectPath, info.getName(), info.getId(), 
				info.getDescription(), info.getMetamodel(), info.getPackage()+"."+info.getClassName());
		try {
			project.refreshLocal(IProject.DEPTH_ZERO, monitor);
			project.refreshLocal(IProject.DEPTH_ONE, monitor);
			project.refreshLocal(IProject.DEPTH_INFINITE, monitor);
			IFile manifest = project.getFile("MANIFEST.MF");
			if(manifest.exists()){
				manifest.refreshLocal(IFile.DEPTH_ZERO, monitor);
				manifest.refreshLocal(IFile.DEPTH_ONE, monitor);
				manifest.refreshLocal(IFile.DEPTH_INFINITE, monitor);
			}
		} catch (CoreException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Starts the search for model smells on the provided root object and searches for the 
	 * <br>- smell activatet for the given <i>IProject</i>
	 * 
	 * @param project - project for which the configured smells are supposed to be found
	 * @param root - root object the search has to be performed on
	 * @param selectedFile 
	 */
	public static void findConfiguredModelSmells(IProject project, EObject root, IFile selectedFile){
		LinkedList<ModelSmell> smells = getSelectedModelSmells(project);
		for(ModelSmell smell : smells){
			if(smell.getFinderClass() instanceof MetricBasedModelSmellFinderClass)
				((MetricBasedModelSmellFinderClass)smell.getFinderClass()).setLimit(getLimit(project, smell.getId()));
		}
		LinkedList<Result> results = ModelSmellFinder.findModelSmells(smells, root);
		//=====
//		String path = root.eResource().getURI().toPlatformString(true);
////		IFile iFile = (IFile)org.eclipse.core.resources.ResourcesPlugin.getWorkspace().getRoot().findMember(path);
//		IFile iFile = selectedFile;
		//=====
		resultModels.add(new ResultModel(results, selectedFile, new Date()));
		if(resultModelViewer!=null && resultModelViewer.getContentProvider() != null)
			if(resultModels != null){
				resultModelViewer.setInput(resultModels);
			}
		for(Result result : results){
			resultsViewInput.addAll(result.getResultViewEntries());
		}
		if(resultsViewer!=null)
			if(resultsViewInput!=null){
				resultsViewer.setInput(resultsViewInput);
			}
	}
	
	/**
	 * Loads a LinkedList of all smell plugins (wrapped into <i>ModelSmell</i> objects) 
	 * <br>- currently installed in the Eclipse instance.
	 * 
	 * @return all smells currently installed
	 */
	public static LinkedList<ModelSmell> getAllModelSmells() {
		if(allSmells==null)
			allSmells = ModelSmellLoader.loadModelSmells();
		return allSmells;
	}
	
	/**
	 * For a given smell ID this method returns the limit set in the configuration. If it 
	 * <br>- is not a metric based smell or the limit was not set. It return 0.0.
	 * @param project - target project
	 * @param smellID - the id of the given smell
	 * @return limit set in the projects configuration or 0.0
	 */
	public static Double getLimit(IProject project, String smellID){
		Configuration config = getConfiguration(project);
		Double limit = config.getLimit(smellID);
		return limit;
	}
	
	
	public static LinkedList<ResultViewEntry> getResultsViewInput(){
		return resultsViewInput;
	}
	
	/**
	 * Return the <i>ModelSmell</i> objects for the smells which are active on the
	 * <br>- given project.
	 * 
	 * @param project - target project
	 * @return - all model smell active for the given project
	 */
	public static LinkedList<ModelSmell> getSelectedModelSmells(IProject project){
		Configuration config = getConfiguration(project);
		return config.getSelectedModelSmells();
	}
	
	/**
	 * Loads stubs for all installed metrics as <i>MetricData</i>.
	 * @return <i>MetricData</i> objects for all installed metric plugins 
	 */
	public static LinkedList<MetricData> loadAllMetricData(){
		return MetricStubLoader.loadModelMetricData();
	}
	
	/**
	 * Loads the <i>Configuration</i> of a target project from its xml-file.
	 * @param project - target project
	 * @return <i>Configuration</i> object for the project
	 */
	public static Configuration loadConfiguration(IProject project) {
		return XMLProjectFile.loadConfigurator(project);
	}
	
	/**
	 * Saves the current configuration of the project by writing it to
	 * <br>- the project's xml-file
	 * @param project - target project
	 */
	public static void saveConfiguration(IProject project) {
		XMLProjectFile.saveConfigurator(getConfiguration(project));
	}

	/**
	 * Sets the configured data for a project by manipulating the 
	 * <br>- <i>Configuration</i> object for this project currently
	 * <br>- held by EMF Smells.
	 * @param project - target project
	 * @param selection - array determening the selection of smells
	 */
	public static void setConfiguration(IProject project, boolean[] selection){
		Configuration configuration= getConfiguration(project);
		configuration.setSelection(selection);
	}
	
	/**
	 * Sets the configured data for a project by manipulating the 
	 * <br>- <i>Configuration</i> object for this project currently
	 * <br>- held by EMF Smells.
	 * <br>- Also sets the limits for metric based smells.
	 * @param project - target project
	 * @param selection - array determening the selection of smells
	 * @param limits - limits for the smells
	 */
	public static void setConfiguration(IProject project, boolean[] selection, double[] limits){
		Configuration configuration= getConfiguration(project);
		configuration.setSelection(selection);
		configuration.setLimits(limits);
	}
	
	public static void setResultModelTreeViewer(ResultModelTreeViewer viewer){
		resultModelViewer = viewer;
	}
	
	public static void setResultsViewer(TableViewer viewer){
		resultsViewer = viewer;
	}
	
	public static LinkedList<ResultModel> getResultModels(){
		return resultModels;
	}
	
	private static Configuration getConfiguration(IProject project) {
		Configuration configuration = null;
		if(!configurations.isEmpty())
		for (Configuration tempConfiguration : configurations) {
			if (tempConfiguration.getProject().equals(project))
				configuration=tempConfiguration;
		}
		if(configuration == null)
			configuration = loadConfiguration(project);
		if(configuration == null){
			configuration = new Configuration(project);	
		}
		configurations.add(configuration);
		return configuration;
	}
	

	
	
	
}
