package de.unimarburg.swt.emf.modelsmell.model;

import java.util.List;

import de.unimarburg.swt.emf.modelsmell.EMFModelSmells;
import de.unimarburg.swt.emf.modelsmell.interfaces.IModelSmellFinderClass;

/**
 * Wrapper class for the single smell plugins.
 * 
 * @author Matthias Burhenne
 *
 */

public class ModelSmell {
	private String id;
	private String name;
	private String description;
	private String metamodel;
	private IModelSmellFinderClass finderClass;

	/**
	 * The constructor that creates a new <i>ModelSmell</i> object.
	 * 
	 * @param name
	 * @param description
	 * @param metamodel
	 * @param calculateClass
	 * @param id
	 */
	public ModelSmell(String name, String description, String metamodel,
			IModelSmellFinderClass finderClass, String id) {
		this.id = id;
		this.name = name;
		this.description = description;
		this.metamodel = metamodel;
		this.finderClass = finderClass;
	}

	public String getId() {
		return id;
	}

	public String getName() {
		return name;
	}

	public String getDescription() {
		if (description == null)
			return " - ";
		return description;
	}

	public String getMetamodel() {
		return metamodel;
	}

	public IModelSmellFinderClass getFinderClass() {
		return finderClass;
	}
	
	public static ModelSmell getSmell(String id){
		List<ModelSmell> smells = EMFModelSmells.getAllModelSmells();
		for(ModelSmell currentSmell : smells){
			if(currentSmell.getId().equals(id))
				return currentSmell;
		}
		return null;
	}

	@Override
	public String toString() {
		return "EMF Model Smell: " + this.name + "(" + this.id + ")";
	}
	
	
}
