package de.unimarburg.swt.emf.modelsmell.popupmenu.action;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.emf.common.ui.viewer.IViewerProvider;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.henshin.interpreter.util.ModelHelper;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.OpenEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IEditorDescriptor;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.internal.WorkbenchPlugin;
import org.eclipse.ui.internal.registry.EditorRegistry;
import org.eclipse.ui.part.FileEditorInput;

import de.unimarburg.swt.emf.modelsmell.EMFModelSmells;

public class FindModelSmellAction implements IObjectActionDelegate {

	private IFile selectedFile;
	private EObject selectedEObject;
	private IProject selectedProject;
	
	private Shell shell;

	public FindModelSmellAction() {
		super();
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
		shell = targetPart.getSite().getShell();
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		try{
			EObject root;
			if(selectedEObject != null){
				root = selectedEObject;
				String path = root.eResource().getURI().toPlatformString(true);
				selectedFile = (IFile)org.eclipse.core.resources.ResourcesPlugin.getWorkspace().getRoot().findMember(path);
			}else{
				root = ModelHelper.loadFile(selectedFile.getFullPath().toString());
			}
			selectedProject = selectedFile.getProject();
			EMFModelSmells.findConfiguredModelSmells(selectedProject, root, selectedFile);
			IWorkbenchWindow workbenchWindow= PlatformUI.getWorkbench().getActiveWorkbenchWindow();
			IWorkbenchPage page = workbenchWindow.getActivePage();
			IDE.openEditor(page, selectedFile);
		}catch(Exception ex){
			ex.printStackTrace();
			Throwable cause = ex.getCause();
			if(!(cause == null) && cause.getClass().getName().equals("org.eclipse.emf.ecore.xmi.PackageNotFoundException")){
				MessageDialog.openError(
						shell,
						"Error when trying to open File",
						"The file you selected is not a (valid) EMF model.");
			}else{
			MessageDialog.openError(
					shell,
					"Error when trying to execute EMF Smell search",
					ex.toString());
			}
		}
		
	}

	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		if(selection instanceof IStructuredSelection){
			Object o = ((IStructuredSelection) selection).getFirstElement();
			if(o instanceof IFile){
				selectedEObject = null;
				selectedFile = (IFile) ((IStructuredSelection)selection).getFirstElement();
			}else if(o instanceof EObject){
				EObject eObject = (EObject)o;
				selectedEObject = eObject;
				String path = eObject.eResource().getURI().toPlatformString(false);
				IFile iFile = (IFile)org.eclipse.core.resources.ResourcesPlugin.getWorkspace().getRoot().findMember(path);
				selectedFile = iFile;
			}
		}
	}

}
