package de.unimarburg.swt.emf.modelsmell.smellfinder;

import java.util.LinkedList;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;

import de.unimarburg.swt.emf.modelsmell.model.ModelSmell;
import de.unimarburg.swt.emf.modelsmell.model.smellfinder.Result;

/**
 * Class handling the execution of a list of <i>ModelSmell</i> objects on a given root elements.
 * <br>- Is a singleton.
 * 
 * @author Matthias Burhenne
 *
 */

public class ModelSmellFinder {
	private static final ModelSmellFinder instance = new ModelSmellFinder();
	//private LinkedList<Result> results = new LinkedList<Result>();
	private static EObject rootElement;
	
	private ModelSmellFinder(){
		
	}
	
	static long timebegin;
	static long timeend;
	
	private static void startTimeRecording() {
		timebegin = System.currentTimeMillis();
	}

	private static void stopTimeRecording() {
		timeend = System.currentTimeMillis();
		System.out.println("Time needed: " + ((timeend - timebegin)) + " ms");
	}
	
	public static ModelSmellFinder instance(){
		return instance;
	}
	
	/**
	 * Starts the identification of the passed smells on the root object and its
	 * <br>- transtitively contained objects.
	 * @param smells - list of <i>ModelSmell</i> objects
	 * @param root - root element of the model under inspection
	 * @return - results of the search
	 */
	public static LinkedList<Result> findModelSmells(LinkedList<ModelSmell> smells, EObject root){
		rootElement = root;
		String nsUri = ((EPackage)root.eClass().eContainer()).getNsURI();
		LinkedList<Result> results = new LinkedList<Result>();
		startTimeRecording();
		for(ModelSmell smell : smells){
			if(smell.getMetamodel().equals(nsUri))
				results.add(findModelSmell(smell));
		}
		stopTimeRecording() ;
		return results;
	}
	
	private static Result findModelSmell(ModelSmell smell){
		LinkedList<LinkedList<EObject>> modelelements = smell.getFinderClass().findSmell(rootElement);
		return new Result(smell, modelelements);
	}

}
