package de.unimarburg.swt.emf.modelsmell.smellfinder.henshin;


import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.henshin.interpreter.EmfEngine;
import org.eclipse.emf.henshin.interpreter.RuleApplication;
import org.eclipse.emf.henshin.interpreter.UnitApplication;
import org.eclipse.emf.henshin.interpreter.interfaces.InterpreterEngine;
import org.eclipse.emf.henshin.interpreter.util.Match;
import org.eclipse.emf.henshin.interpreter.util.ModelHelper;
import org.eclipse.emf.henshin.matching.EmfGraph;
import org.eclipse.emf.henshin.model.Rule;
import org.eclipse.emf.henshin.model.TransformationSystem;
import org.eclipse.emf.henshin.model.TransformationUnit;

/**
 * Class handling the execution of smell plugins based on EMF Henshin.	
 * 
 * @author Matthias Burhenne
 *
 */
	
public class HenshinRunner {

	//private static final String MAINUNIT = "mainUnit";
	public static final String MAINRULE = "mainRule";
	public static final String MODELELEMENT_NAME = "modelElementName";
	private static final String PRERUNIT = "preUnit";
	private static final String POSTUNIT = "postUnit";
	
	
	private static TransformationSystem transformationSystem;

	public HenshinRunner(){
	}
	
	/**
	 * This returns a list of EObjects for every match it finds. The eObjects are the ones identified for the LHS
	 * @param rootElement - EObject (root of the considered model)
	 * @param henshinFilePath - String (Path to the HenshinFile on file system)
	 * @return list of list of <i>EObjects</i>
	 */
	public static LinkedList<LinkedList<EObject>> run(EObject rootElement, String henshinFilePath){
		LinkedList<LinkedList<EObject>> result = new LinkedList<LinkedList<EObject>>();
		EObject root = rootElement;
		String path = henshinFilePath;
		transformationSystem = (TransformationSystem) ModelHelper.loadFile(path);
		EmfGraph graph = new EmfGraph();
		graph.addRoot(root);
		InterpreterEngine engine = new EmfEngine(graph);
		Rule checkerRule = transformationSystem.findRuleByName(MAINRULE);
		TransformationUnit preUnit = transformationSystem.findUnitByName(PRERUNIT);
		TransformationUnit postUnit = transformationSystem.findUnitByName(POSTUNIT);
		RuleApplication ruleApp = new RuleApplication(engine, checkerRule);
		List<Match> allMatches = ruleApp.findAllMatches();
		if(preUnit != null){
			UnitApplication preApp = new UnitApplication(engine, preUnit);
			preApp.execute();
		}
		for(Match match : allMatches){
			LinkedList<EObject> eObjects = new LinkedList<EObject>();
			eObjects.addAll(match.getNodeMapping().values());
			result.add(eObjects);
		}
		if(postUnit != null){
			UnitApplication postApp = new UnitApplication(engine, postUnit);
			postApp.execute();
		}
		return  result;
	}
	
//		public static LinkedList<String> run(EObject rootElement, String henshinFilePath){
//			LinkedList<String> result = new LinkedList<String>();
//			EObject root = rootElement;
//			String path = henshinFilePath;
//			transformationSystem = (TransformationSystem) ModelHelper.loadFile(path);
//			EmfGraph graph = new EmfGraph();
//			graph.addRoot(root);
//			InterpreterEngine engine = new EmfEngine(graph);
//			Rule checkerRule = transformationSystem.findRuleByName(MAINRULE);
//			TransformationUnit preUnit = transformationSystem.findUnitByName(PRERUNIT);
//			TransformationUnit postUnit = transformationSystem.findUnitByName(POSTUNIT);
//			RuleApplication ruleApp = new RuleApplication(engine, checkerRule);
//			List<Match> allMatches = ruleApp.findAllMatches();
//			if(preUnit != null){
//				UnitApplication preApp = new UnitApplication(engine, preUnit);
//				preApp.execute();
//			}
//			for(Match match : allMatches){
//				Parameter param = checkerRule.getParameterByName(MODELELEMENT_NAME);
//				String name = match.getParameterValues().get(param).toString();
//				result.add(name);
//			}
//			if(postUnit != null){
//				UnitApplication postApp = new UnitApplication(engine, postUnit);
//				postApp.execute();
//			}
//			return  result;
//		}
}
