package de.unimarburg.swt.emf.modelsmell.util;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.jar.Attributes;
import java.util.jar.Manifest;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;

import de.unimarburg.swt.emf.modelsmell.model.generator.MetricBasedModelSmellInfo;
import de.unimarburg.swt.emf.modelsmell.model.generator.ModelSmellInfo;

/**
 * Helper class for updating the dependencies of the target project of
 * <br>- a smell plugin generation.
 * 
 * @author Matthias Burhenne
 * @author Pawel Stepien
 *
 */

public class DependenciesFile{

	private final static String ECORE = "org.eclipse.emf.ecore";
	private final static String EMF_MODELSMELLS = "de.unimarburg.swt.emf.modelsmell";
	private final static String EMF_METRICS = "de.unimarburg.swt.emf.metrics";
	private final static String REQ_BUNDLE = "Require-Bundle";
	private final static String MANIFEST_FILE = "/META-INF/MANIFEST.MF";
	
	/**
	 * Sets the dependencies of the target project by manipulating the MANIFEST.MF of
	 * <br>- the project accordingly.
	 * @param projectName - name of the target project
	 * @param info - the  <i>ModelSmellInfo</i> containing the data for the new smell
	 */
	public static void updateDependencies(String projectName, ModelSmellInfo info) {
		IProject project = ResourcesPlugin.getWorkspace().getRoot().getProject(projectName);
		try {
			URL newUrl = new URL(project.getLocationURI().toURL().toString()
					+ MANIFEST_FILE);
			InputStream is = newUrl.openStream();
			Manifest mf = new Manifest(is);
			Attributes att = mf.getMainAttributes();
			String value = att.getValue(REQ_BUNDLE);
			if (!value.contains(ECORE))
				value = value + "," + ECORE;
			if (!value.contains(EMF_MODELSMELLS))
				value = value + "," + EMF_MODELSMELLS;
			if((info instanceof MetricBasedModelSmellInfo) && !value.contains(EMF_METRICS))
				value = value + "," + EMF_METRICS;
			att.putValue(REQ_BUNDLE, value);
			FileOutputStream out = new FileOutputStream(newUrl.getFile());
			mf.write(out);
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}
