package de.unimarburg.swt.emf.modelsmell.util;

import java.util.LinkedList;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;


import de.unimarburg.swt.emf.modelsmell.extensionpoint.ExtensionPoint;
import de.unimarburg.swt.emf.modelsmell.interfaces.IModelSmellFinderClass;
import de.unimarburg.swt.emf.modelsmell.model.ModelSmell;

/**
 * Loader class for the installed EMF Smells plugins. It wraps the plugins into
 * <br>- <i>ModelSmell</i> objects which are then used by EMF Smells.
 * 
 * @author Matthias Burhenne
 *
 */

public class ModelSmellLoader {
	
	/**
	 * Loads all smell plugins currently installed an returns them
	 * <br>- as a list of <i>ModelSmell</i> objects.
	 * @return list of <i>ModelSmell</i> objects wrapping all currently installed smell plugins
	 */
	public static LinkedList<ModelSmell> loadModelSmells() {
		LinkedList<ModelSmell> smells = new LinkedList<ModelSmell>();
		IConfigurationElement[] rawSmells = Platform.getExtensionRegistry()
				.getConfigurationElementsFor(ExtensionPoint.EXTENSION_POINT_NAME);
		for (IConfigurationElement element : rawSmells) {
			try {
				if (element.getName().equals(ExtensionPoint.MODELSMELL_TAG)) {					
					ModelSmell smell = createModelSmell(element);
					if (null != smell) {
						smells.add(smell);
					}
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return smells;
	}
	
	private static ModelSmell createModelSmell(IConfigurationElement element){
		try {
			final String name = element.getAttribute(ExtensionPoint.MODELSMELL_NAME_TAG);
			final String id = element.getAttribute(ExtensionPoint.MODELSMELL_ID_TAG);
			final String description = element
					.getAttribute(ExtensionPoint.MODELSMELL_DESCRIPTION_TAG);
			final String metamodel = element
					.getAttribute(ExtensionPoint.MODELSMELL_METAMODEL_TAG);
			final IModelSmellFinderClass finderClass = (IModelSmellFinderClass)element.createExecutableExtension(ExtensionPoint.MODELSMELL_FINDER_CLASS_TAG);
			return new ModelSmell(name, description, metamodel, finderClass, id);
		} catch (Throwable e) {
			e.printStackTrace();
			return null;
		}
	}
	
	public static IConfigurationElement[] getExtensions(){
		return Platform.getExtensionRegistry()
		.getConfigurationElementsFor(ExtensionPoint.EXTENSION_POINT_NAME);
	}
}
