package de.unimarburg.swt.emf.modelsmell.util.xml;

import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.emf.ecore.EObject;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import de.unimarburg.swt.emf.modelsmell.model.ModelSmell;
import de.unimarburg.swt.emf.modelsmell.model.smellfinder.Result;
import de.unimarburg.swt.emf.modelsmell.model.smellfinder.ResultViewEntry;

/**
 * Class for writing the search results to an xml file. Currently used by the old
 * <br>- ModelSmellResultsView.
 * @author Pawel Stepien
 * @author Matthias Burhenne
 *
 */

public class XMLResultsFile extends XMLFile{

	private static final String RESULTS_TAG = "results";
	private static final String IDENTIFIED_SMELL_TAG = "identified_smell";
	private static final String IDENTIFIED_SMELL_TIME_TAG = "time";
	private static final String IDENTIFIED_SMELL_MODELELEMENT_TAG = "model_element";
	
	public static void saveResults(String path, List<ResultViewEntry> results){
		final DocumentBuilder builder = createDocumentBuilder();
		if (builder != null){
			final Document doc = builder.newDocument();		
			final Element root = doc.createElement(RESULTS_TAG);
			doc.appendChild(root);	
//			createAllContextElements(root, doc, results);
			createResultFile(root, doc, results);
			final Transformer transformer = createTransformer();
			final DOMSource source = new DOMSource(doc);		
			final StreamResult result = new StreamResult(path);			
			try {
				transformer.transform(source, result);
			} catch (final TransformerException e) {
				e.printStackTrace();
			}		
		}
	}

	private static void createResultFile(Element root, Document doc,
			List<ResultViewEntry> results) {
		HashMap<ModelSmell, LinkedList<ResultViewEntry>> sortedResults = new HashMap<ModelSmell, LinkedList<ResultViewEntry>>();
		for(ResultViewEntry entry : results){
			if(sortedResults.containsKey(entry.getSmell())){
				sortedResults.get(entry.getSmell()).add(entry);
			}else{
				LinkedList<ResultViewEntry> value = new LinkedList<ResultViewEntry>();
				value.add(entry);
				sortedResults.put(entry.getSmell(), value);
			}
		}
		for(ModelSmell smell : sortedResults.keySet()){
			for(ResultViewEntry entry : sortedResults.get(smell)){
				Element resultEntry = doc.createElement(IDENTIFIED_SMELL_TAG);
				resultEntry.setAttribute(MODELSMELL_ID_TAG, entry.getSmell().getId());
				resultEntry.setAttribute(IDENTIFIED_SMELL_TIME_TAG, entry.getTimeStamp());
				resultEntry.setAttribute(IDENTIFIED_SMELL_MODELELEMENT_TAG, entry.getModelelement());
				root.appendChild(resultEntry);
			}
		}
		
	}
	
	
	
	
//	private static void createAllContextElements(Element root, Document doc, List<Result> results){
//		List<List<EObject>> allContexts = new LinkedList<List<EObject>>();
//		for(Result result : results)
//			if(!allContexts.contains(result.getContext()))
//				allContexts.add(result.getContext());
//		for(List<EObject> context : allContexts)
//			root.appendChild(createContextElement(doc, context, filterResults(results, context)));
//	}
	
//	private static Element createContextElement(Document doc, List<EObject> cont, List<Result> results){
//		final Element context = doc.createElement("context");
//		final Element contextValue = doc.createElement("contextValue");
//		contextValue.setTextContent(cont.get(0).toString());
//		context.appendChild(contextValue);
//		for (Object rawResult : results){
//			Result result = (Result)rawResult;
//			Element resultEntry = doc.createElement("metricResult");
//			Element dateStamp = doc.createElement("date");
//			dateStamp.setTextContent(result.getTimeStamp());
//			resultEntry.appendChild(dateStamp);
//			Element smellEntry = doc.createElement(MODELSMELL_TAG);
//			Element smellName = doc.createElement(MODELSMELL_NAME_TAG);
//			smellName.setTextContent(result.getSmell().getName());
//			smellEntry.appendChild(smellName);
//			Element smellDescription = doc.createElement("metricDescription");
//			smellDescription.setTextContent(result.getSmell().getDescription());
//			smellEntry.appendChild(smellDescription);
//			resultEntry.appendChild(smellEntry);
//			Element resultValue = doc.createElement("resultValue");
//			resultValue.setTextContent(""+result.getResultValue());
//			resultEntry.appendChild(resultValue);
//			context.appendChild(resultEntry);
//		}
//		return context;
//	}
//	
//	private static List<Result> filterResults(List<Result> results, List<EObject> context){
//		List<Result> filteredResults = new LinkedList<Result>();
//		for(Result result : results)
//			if(result.getContext().equals(context))
//				filteredResults.add(result);
//		return filteredResults;
//	}
//	
}
