package de.unimarburg.swt.emf.modelsmell.util.xml;

import java.io.File;
import java.io.IOException;
import java.util.LinkedList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;


import de.unimarburg.swt.emf.modelsmell.model.ModelSmell;
import de.unimarburg.swt.emf.modelsmell.model.preferences.SmellGroup;

/**
 * Utility class for the access of xml-files for the definition and configuration of
 * <br>- smell groups. Currently not used since smell groups are not yet supported.
 * 
 * @author Matthias Burhenne
 *
 */

public class XMLSmellGroupsConfigFile extends XMLFile {
	private static final String SMELL_GROUP_FILENAME = "smellgroups.xml";
	private static final String CONFIGURATOR_TAG = "smellGroupsConfiguration";
	private static final String SMELL_GROUP_TAG = "smellGroup";
	
	private static final String SMELL_GROUP_ID_TAG = "id";
	private static final String SMELL_GROUP_NAME_TAG = "name";
	
	private static final String MODELSMELL_ID_TAG ="id";
	
	
	private final String path;
	private final File configFile;
	
	public XMLSmellGroupsConfigFile(String path){
		this.path = path;
		this.configFile = loadOrCreateSmellGroupConfigFile();
	}

	private File loadOrCreateSmellGroupConfigFile() {
		File file = new File(path + SMELL_GROUP_FILENAME);
		boolean exists = file.exists();
		if(!exists){
			final DocumentBuilder builder = createDocumentBuilder();
			if (builder != null){
				final Document doc = builder.newDocument();		
				final Element root = doc.createElement(CONFIGURATOR_TAG);
				doc.appendChild(root);					
				final Transformer transformer = createTransformer();
				final DOMSource source = new DOMSource(doc);		
				StreamResult result = new StreamResult(this.path + SMELL_GROUP_FILENAME);
				try {
					transformer.transform(source, result);
				} catch (final TransformerException e) {
					e.printStackTrace();
				}		
			}
		}
		return file;
	}
	
	public void writeSmellGroupToConfigFile(SmellGroup smellGroup){
		final DocumentBuilder builder = createDocumentBuilder();
		if (builder != null){
			Document doc = null;
			try {
				doc = builder.parse(configFile);
			} catch (final SAXException e) {
				e.printStackTrace();
			} catch (final IOException e) {
				e.printStackTrace();
			}
			Element oldGroupElement = null;
			NodeList groups = doc.getElementsByTagName(SMELL_GROUP_TAG);
			for(int i = 0; i < groups.getLength(); i++){
				if((((Element)(groups.item(i))).getAttribute(SMELL_GROUP_ID_TAG)).equals(smellGroup.getId())){
						oldGroupElement = (Element) groups.item(i);
						oldGroupElement.getParentNode().removeChild(oldGroupElement);
				}
			}
			
			Element groupElement = doc.createElement(SMELL_GROUP_TAG);
			Element root = doc.getDocumentElement();	
			groupElement.setAttribute(SMELL_GROUP_ID_TAG, smellGroup.getId());
			groupElement.setAttribute(SMELL_GROUP_NAME_TAG, smellGroup.getName());
			root.appendChild(groupElement);
			
			for(ModelSmell smell : smellGroup.getSmells()){
				Element smellElement = doc.createElement(MODELSMELL_TAG);
				smellElement.setAttribute(MODELSMELL_ID_TAG, smell.getId());
				groupElement.appendChild(smellElement);
			}
			final Transformer transformer = createTransformer();
			final DOMSource source = new DOMSource(doc);		
			StreamResult result = new StreamResult(configFile);			
			try {
				transformer.transform(source, result);
			} catch (final TransformerException e) {
				e.printStackTrace();

			}
//				NodeList entryList = root.getElementsByTagName(METRIC_TAG);
//				LinkedList<String> metricIdList = new LinkedList<String>();
//				for (int i=0; i < entryList.getLength(); i++){
//					Element entry = (Element) entryList.item(i);
//					NodeList nodeList = entry.getElementsByTagName(METRIC_ID_TAG);
//					Element metricId = (Element) nodeList.item(0);
//					metricIdList.add(metricId.getTextContent());
//				}	
//				configurator.setSelection(metricIdList);
//				}	
	}
	}
	
	private Element getOldGroupElement(Document doc, String groupID){
		NodeList groups = doc.getElementsByTagName(SMELL_GROUP_TAG);
		for(int i = 0; i < groups.getLength(); i++){
			if((((Element)(groups.item(i))).getAttribute(SMELL_GROUP_ID_TAG)).equals(groupID))
					return (Element) groups.item(i);
		}
		return null;
	}
	
}
