package de.unimarburg.swt.emf.modelsmell.views;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.ui.viewer.IViewerProvider;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.edit.domain.IEditingDomainProvider;
import org.eclipse.emf.henshin.interpreter.util.ModelHelper;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorDescriptor;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorRegistry;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.part.ISetSelectionTarget;
import org.eclipse.ui.part.ViewPart;

import de.unimarburg.swt.emf.modelsmell.EMFModelSmells;
import de.unimarburg.swt.emf.modelsmell.model.smellfinder.treeviewer.EObjectGroup;
import de.unimarburg.swt.emf.modelsmell.model.smellfinder.treeviewer.ResultModel;

/**
 * View for displaying the <i>ResultModel</i> of a smell identification process.
 * 
 * @author Matthias Burhenne
 *
 */

public class ResultModelTreeView extends ViewPart {
	private static final String MENU_ID = "de.unimarburg.swt.emf.modelsmell.resultview.contextmenu";
	private ResultModelTreeViewer viewer;
	private Composite parent;
	
	
	@Override
	public void createPartControl(Composite parent) {
		this.parent = parent;
		this.viewer = new ResultModelTreeViewer(parent); //TODO: add link between viewer and EMFModelSmells
		viewer.setContentProvider(new ResultModelTreeViewerContentProvider());
		viewer.setLabelProvider(new ResultModelTreeViewerLabelProvider());
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {
			
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				 if(event.getSelection().isEmpty()) {
			           return;
			     }
				 IEditorPart editorPart = null;
				 if(event.getSelection() instanceof IStructuredSelection) {
					 editorPart = openEditor((IStructuredSelection)event.getSelection());
				 }
				 if(editorPart != null && editorPart instanceof IViewerProvider){
					 setSelectionInModel((IViewerProvider)editorPart, (IStructuredSelection)event.getSelection());
//						 ((IViewerProvider)editorPart).getViewer().setSelection(event.getSelection(), true);
				 }
			}
			
			private void setSelectionInModel(IViewerProvider provider, IStructuredSelection selection){
				ArrayList<EObject> eObjectList = new ArrayList<EObject>();
				for(Object selectedObject : selection.toList()){
					if(selectedObject instanceof EObjectGroup){
						for(EObject currentEObject : ((EObjectGroup)selectedObject).getEObjects()){
							EObject modelInstanceObject = getModelObjectInstance(provider, currentEObject);
							eObjectList.add(modelInstanceObject);
						}
						((EObjectGroup)selectedObject).setEObjects(eObjectList);
					}else if(selectedObject instanceof EObject){
						eObjectList.add(getModelObjectInstance(provider, (EObject)selectedObject));
					}
				}
				provider.getViewer().setSelection(new StructuredSelection(eObjectList));
//				Object o = selection.getFirstElement();
//				if(o instanceof EObject){
//					Object input = provider.getViewer().getInput();
//					provider.getViewer().setSelection(selection);
//				}else if(o instanceof EObjectGroup){
//					StructuredSelection s = new StructuredSelection(((EObjectGroup)o).getEObjects());
//					provider.getViewer().setSelection(s);
//				}
				
			}
			
			private EObject getModelObjectInstance(IViewerProvider provider, EObject eObject) {
				Viewer viewer = provider.getViewer();
				TreeViewer treeViewer = (TreeViewer)viewer;
				Tree tree = treeViewer.getTree();
				TreeItem treeItem = tree.getItem(0);
				Resource o = (Resource)treeItem.getData();
				ResourceSet set = o.getResourceSet();
				EObject ob = set.getEObject(EcoreUtil.getURI(eObject), true);
				String uri = o.getURIFragment(eObject);
				return ob;
//				return o.getEObject(uri);
//				EObject equalObject = o.getEObject(eObject.eResource().getURIFragment(eObject));
//				return equalObject;
				

				
//				if(provider instanceof IEditingDomainProvider){
//					IEditingDomainProvider prov = (IEditingDomainProvider)provider;
//					TreeIterator iterator = prov.getEditingDomain().treeIterator(viewer.getCurrentRoot());
//					while(iterator.hasNext()){
//						Object itObj = iterator.next();
//						if(itObj.equals(eObject)){
//							return (EObject) itObj;
//						}
//					}
//				}
//				return eObject;
			}

			private IEditorPart openEditor(IStructuredSelection selection){
				Object o = selection.getFirstElement();
				EObject obj = null;
				if(o instanceof EObject){
					obj = (EObject)o;
				}else if(o instanceof EObjectGroup){
					EObjectGroup group = (EObjectGroup)o;
					if(!group.getEObjects().isEmpty()){
						obj = group.getEObjects().getFirst();
					}
				}
				if(obj != null){
//					URI uri = obj.eResource().getURI();
//					String fileString = URI.decode(uri.path());
//					fileString = fileString.replaceFirst("/resource", "");
//					IFile file = ResourcesPlugin.getWorkspace().getRoot().getFile(new Path(fileString));
//					IWorkbench workbench = PlatformUI.getWorkbench();
					IFile file = ((LinkedList<ResultModel>)viewer.getInput()).get(0).getIFile();
					IWorkbenchWindow workbenchWindow= PlatformUI.getWorkbench().getActiveWorkbenchWindow();
					IWorkbenchPage page = workbenchWindow.getActivePage();
					try {
						if(file.exists()){
							return IDE.openEditor(page, file);
						}
					}catch (PartInitException exception) {
						return null;
					}
				}
		        return null;
		     }
				
			
		});
		EMFModelSmells.setResultModelTreeViewer(viewer);
		LinkedList<ResultModel> result = EMFModelSmells.getResultModels();
		if(result != null){
			viewer.setInput(result);
		}
		Action clearAction = new Action(){
			public void run() {
				EMFModelSmells.clearResultModels();
			}
		};
		clearAction.setText("Clear Results");
		clearAction.setToolTipText("Remove all results from the list");
		clearAction.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(ISharedImages.IMG_ETOOL_DELETE));
		IActionBars bars = getViewSite().getActionBars();
		bars.getToolBarManager().add(clearAction);
		
		createContextMenu();
	}
	

	private void createContextMenu() {
		MenuManager menuMgr = new MenuManager();
        menuMgr.setRemoveAllWhenShown(true);
        menuMgr.addMenuListener(new IMenuListener() {
                     

			@Override
			public void menuAboutToShow(IMenuManager manager) {
				fillContextMenu(manager);			
			}
			
			private void fillContextMenu(IMenuManager manager){
				 manager.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
			}
        });
             
             // Create menu.
          Menu menu = menuMgr.createContextMenu(viewer.getControl());
             viewer.getControl().setMenu(menu);
             
             // Register menu for extension.
          getSite().registerContextMenu(MENU_ID, menuMgr, viewer);
    }
		



	@Override
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	
	

}
