package de.unimarburg.swt.emf.modelsmell.wizards.pages;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;

import org.eclipse.core.resources.IProject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;

import de.unimarburg.swt.emf.modelsmell.EMFModelSmells;
import de.unimarburg.swt.emf.modelsmell.model.ModelSmell;
import de.unimarburg.swt.emf.modelsmell.model.generator.GenerationType;
import de.unimarburg.swt.emf.modelsmell.util.xml.XMLPluginFile;
import de.unimarburg.swt.emf.modelsmell.wizards.NewSmellWizard;

/**
 * The wizard page requesting the basic data needed for the generation of every smell plugin.
 * <br>- It is the starting page of the wizard and determines the following pages.
 * 
 * @author Matthias Burhenne
 *
 */

public class ModelSmellDataPage extends WizardPage implements Listener {

	public static final String GENERATE_CODE_SKELETON_OPTION_LABEL = "Generate Code Skeleton";
	public static final String GENERATE_MODEL_SMELL_FROM_HENSHIN_FILE_OPTION_LABEL ="Generate Modelsmell from Henshin file";
	public static final String GENERATE_MODEL_SMELL_FROM_METRIC_OPTION_LABEL ="Generate Modelsmell from Metric";
	
	private static final String PAGE_NAME = "de.unimarburg.swt.emf.modelsmell.wizards.pages.ModelSmellDataPage";
	private static final String TITLE = "Model Smell Data";
	private static final String DESCRIPTION = "Enter the required Data for the Generation of the new Model Smell.\n"
			+ "(required fields are denoted by \"*\")";
	private Combo projectCombo;
	private Text nameTextField;
	private Text idTextField;
	private Text descriptionTextField;
	private Combo metamodelCombo;

	private Combo sourceCombo;
	private boolean initialization = false;
	
	public ModelSmellDataPage(){
		super(PAGE_NAME);
		setTitle(TITLE);
		setDescription(DESCRIPTION);
	}
	
	@Override
	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		final GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		container.setLayout(layout);
		createTextFields(container);
		initContents();
		setControl(container);
		this.setPageComplete(false);

	}
	
	private void initContents() {
		initialization=true;
		initProjectsAndMetamodels();
		//initFromSelectedEObject();
		initialization=false;
	}
	
	private void initProjectsAndMetamodels(){
		for (IProject project : ((NewSmellWizard)getWizard()).getProjects()) {
			projectCombo.add(project.getName());
		}
		Object [] metamodelObjects = EPackage.Registry.INSTANCE.keySet().toArray(new Object[EPackage.Registry.INSTANCE.size()]);
        Arrays.sort(metamodelObjects);
		for(Object object : metamodelObjects){
			metamodelCombo.add(object.toString());
		}
	}

	private void createTextFields(Composite container) {
		Label label;
		GridData gridData;
		Group group;
		GridLayout layout;
		// - Project -
		group = new Group(container, SWT.NONE);
		group.setText("Project");
	    layout = new GridLayout();
	    layout.numColumns = 2;
	    group.setLayout(layout);
	    gridData = new GridData(GridData.FILL_HORIZONTAL);
	    group.setLayoutData(gridData);
		// col:1
		label = new Label(group, SWT.NONE);
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		label.setText("Target project *:");
		// col:2
		projectCombo = new Combo(group, SWT.READ_ONLY);
		final GridData projectComboData = new GridData(GridData.FILL_HORIZONTAL);
		projectCombo.setLayoutData(projectComboData);
		//projectCombo.setText("Select Project");
		projectCombo.addListener(SWT.Selection, (org.eclipse.swt.widgets.Listener)this);
		// - Modelsmell Data -
		group = new Group(container, SWT.NONE);
		group.setText("Model Smell data");
	    layout = new GridLayout();
	    layout.numColumns = 2;
	    group.setLayout(layout);
	    gridData = new GridData(GridData.FILL_HORIZONTAL);
	    group.setLayoutData(gridData);
		// - Name -
		// col:1
		label = new Label(group, SWT.NONE);
		label.setText("Name *:");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		// col:2
		nameTextField = new Text(group, SWT.BORDER);
		nameTextField.addListener(SWT.Modify, this);
		nameTextField.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		// - Id -
		// col:1
		label = new Label(group, SWT.NONE);
		label.setText("Id *:");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		// col:2
		idTextField = new Text(group, SWT.BORDER);
		idTextField.addListener(SWT.Modify, this);
		idTextField.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		// - Description -
		// col:1
		label = new Label(group, SWT.NONE);
		label.setText("Description:");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		// col:2
		descriptionTextField = new Text(group, SWT.BORDER);
		descriptionTextField.addListener(SWT.Modify, this);
		descriptionTextField.setLayoutData(new GridData(
				GridData.FILL_HORIZONTAL));
		// - Metamodel -
		// col:1
		label = new Label(group, SWT.NONE);
		label.setText("Metamodel *:");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		// col:2
		
		metamodelCombo = new Combo(group, SWT.READ_ONLY);
		final GridData metamodelComboData = new GridData(GridData.FILL_HORIZONTAL);
		metamodelCombo.setLayoutData(metamodelComboData);
		//projectCombo.setText("Select Project");
		metamodelCombo.addListener(SWT.Selection, (org.eclipse.swt.widgets.Listener)this);
		// - Source -
		group = new Group(container, SWT.NONE);
		group.setText("Generation Options");
	    layout = new GridLayout();
	    layout.numColumns = 2;
	    group.setLayout(layout);
	    gridData = new GridData(GridData.FILL_HORIZONTAL);
	    group.setLayoutData(gridData);
		// col:1
		label = new Label(group, SWT.NONE);
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		label.setText("Source *:");
		// col:2
		sourceCombo = new Combo(group, SWT.READ_ONLY);
		final GridData comboData = new GridData(GridData.FILL_HORIZONTAL);
		sourceCombo.setLayoutData(comboData);
		sourceCombo.setItems(new String[] { GENERATE_CODE_SKELETON_OPTION_LABEL, 
				GENERATE_MODEL_SMELL_FROM_HENSHIN_FILE_OPTION_LABEL,
				GENERATE_MODEL_SMELL_FROM_METRIC_OPTION_LABEL});
		sourceCombo.setText("Select generation type");
		sourceCombo.addListener(SWT.Selection, this);
		
	}



	@Override
	public void handleEvent(Event event) {
		if (event.widget == sourceCombo){
			GenerationType type = GenerationType.SKELETON;
			if(sourceCombo.getText().equals(GENERATE_CODE_SKELETON_OPTION_LABEL))
				type = GenerationType.SKELETON;
			if(sourceCombo.getText().equals(GENERATE_MODEL_SMELL_FROM_HENSHIN_FILE_OPTION_LABEL))
				type = GenerationType.HENSHIN_TRANSFORMATION;	
			if(sourceCombo.getText().equals(GENERATE_MODEL_SMELL_FROM_METRIC_OPTION_LABEL))
				type = GenerationType.METRIC_BASED;	
			((NewSmellWizard)getWizard()).setGenerationType(type);
		}
		if (event.widget == projectCombo)
			((NewSmellWizard)getWizard()).setNewSmellTargetProject(projectCombo.getText());
		if(!initialization ){
			updatePageComplete();
			getWizard().getContainer().updateButtons();		
		}
		
	}
	
	@Override
	public WizardPage getNextPage() {
		
		if (((NewSmellWizard)getWizard()).getGenerationType() == GenerationType.HENSHIN_TRANSFORMATION) {
			HenshinDataWizardPage page = ((NewSmellWizard)getWizard()).getHenshinPage();
			initialization = true;
			((NewSmellWizard)getWizard()).getHenshinPage().initContents();
			initialization = false;
			return page;
		}
		if (((NewSmellWizard)getWizard()).getGenerationType() == GenerationType.METRIC_BASED) {
			MetricBasedDataWizardPage page = ((NewSmellWizard)getWizard()).getMetricPage();
			initialization = true;
			((NewSmellWizard)getWizard()).getMetricPage().initContents();
			initialization = false;
			return page;
		}
		return null;
	}

	/**
	 * Wird jedes mal ausgefhr wenn sich der Inhalt eines Textfeldes im Wizard
	 * verndert. berprft die inhalte der Textfelder und erzeugt
	 * entschprechende Meldungen im Wizardfenster.
	 */
	private void updatePageComplete() {

		if(nameTextField.getText().isEmpty()){
			this.setMessage("Modelsmell name is not specified.", ERROR);
			this.setPageComplete(false);
		}else
		if(idTextField.getText().isEmpty()){
			this.setMessage("Modelsmell ID is not specified.", ERROR);
			this.setPageComplete(false);
		}else
		if(!checkNewID()){
			this.setMessage("The ID you provided is already in use", ERROR);
			this.setPageComplete(false);
		}else
		if(metamodelCombo.getText().isEmpty()){
			this.setMessage("Modelsmell metamodel is not specified.", ERROR);
			this.setPageComplete(false);
		}else
		if(!checkProject()){
			this.setMessage("Target project for the Modelsmell is not specified.", ERROR);
			this.setPageComplete(false);
		}else
		if(!checkGenerationType()){
			this.setMessage("Generation type for the Modelsmell is not specified.", ERROR);
			this.setPageComplete(false);
		}else{
			this.setMessage("");
			saveTextFieldValues();
			this.setPageComplete(true);
			
		}
	}
	
	private boolean checkNewID(){
		ArrayList<String> usedIDs = new ArrayList<String>();
		for(ModelSmell smell : EMFModelSmells.getAllModelSmells()){
			usedIDs.add(smell.getId());
		}
		LinkedList<IProject> pluginProjects = ((NewSmellWizard)this.getWizard()).getProjects();
		for(IProject project : pluginProjects){
			usedIDs.addAll(XMLPluginFile.readSmellIDs(project));
		}
		return !usedIDs.contains(this.idTextField.getText());

	}

	private boolean checkGenerationType() {
		return sourceCombo.getText().equals(GENERATE_CODE_SKELETON_OPTION_LABEL) 
			|| sourceCombo.getText().equals(GENERATE_MODEL_SMELL_FROM_HENSHIN_FILE_OPTION_LABEL)
			|| sourceCombo.getText().equals(GENERATE_MODEL_SMELL_FROM_METRIC_OPTION_LABEL);
	}

	private void saveTextFieldValues() {
		((NewSmellWizard)getWizard()).setNewSmellId(this.idTextField.getText());
		((NewSmellWizard)getWizard()).setNewSmellName(this.nameTextField.getText()); 
		((NewSmellWizard)getWizard()).setNewSmellDescription(this.descriptionTextField.getText()); 
		((NewSmellWizard)getWizard()).setNewSmellMetamodel(this.metamodelCombo.getText()); 
	}
	
	private boolean checkProject(){
		if(!projectCombo.getText().isEmpty())
			return true;
		return false;
	}

	@Override
	public boolean isPageComplete() {
		if(initialization)
			return false;
		return super.isPageComplete();
	}
	
	
}
