/*******************************************************************************
 * Copyright (c) Philipps University of Marburg. All rights reserved. 
 * This program and the accompanying materials are made 
 * available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Philipps University of Marburg - initial API and implementation
 *******************************************************************************/
package org.eclipse.emf.refactor.common.core;

import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.ltk.core.refactoring.participants.RefactoringProcessor;

/**
 * Interface used for specifying the controller of a specific model refactoring.
 * @generated NOT
 * @author Thorsten Arendt
 */
public interface IController {
	
	/**
	 * Sets the EmfRefactoring supported by the controller.
	 * @param emfRefactoring EmfRefactoring supported by the controller.
	 */
	public void setParent(EmfRefactoring emfRefactoring);
	
	/**
	 * Gets the EmfRefactoring supported by the controller.
	 * @return EmfRefactoring supported by the controller.
	 */
	public EmfRefactoring getParent();
	
	/**
	 * Sets the selected EObject (invocation context of the model refactoring).
	 * @param selection Invocation context of the model refactoring.
	 */
	public void setSelection(List<EObject> selection);

	/**
	 * Returns the DataManagement object of the model refactoring.
	 * @return DataManagement object of the model refactoring.
	 */
	public IDataManagement getDataManagementObject();
	
	
	/**
	 * Returns the ltk RefactoringProcessor of the model refactoring.
	 * @return Ltk RefactoringProcessor of the model refactoring.
	 */
	public RefactoringProcessor getLtkRefactoringProcessor();

}
