/*******************************************************************************
 * Copyright (c) Philipps University of Marburg. All rights reserved. 
 * This program and the accompanying materials are made 
 * available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Philipps University of Marburg - initial API and implementation
 *******************************************************************************/
package org.eclipse.emf.refactor.common.groupmanagement.ui;

import org.eclipse.emf.refactor.common.config.EmfRefactorConfig;
import org.eclipse.emf.refactor.common.groupmanagement.Group;
import org.eclipse.emf.refactor.common.groupmanagement.GroupManagementController;
import org.eclipse.emf.refactor.common.groupmanagement.GroupManagementModus;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

/**
 * PreferencePage of EMF Refactor.
 * @generated NOT
 * @author Thorsten Arendt
 */
public class GroupManagementPreferencePage extends PreferencePage implements
		IWorkbenchPreferencePage {
	
	/**
	 * Controller of the group management process.
	 */
	private GroupManagementController gmc;
	
	private Composite menuWidget;
	private Composite selectionWidget;
	private Button addButton, updateButton, showButton, deleteButton, 
												renameButton, menuOkButton;
	private Button selectionCancelButton, selectionOkButton;
	private Combo combo;
	private Composite nameWidget;
	private Text textName;
	private Button nameCancelButton, nameOkButton;
	
	/**
	 * Default contructor that initializes the controller of the 
	 * group management process.
	 */
	public GroupManagementPreferencePage() {
		super();
		noDefaultAndApplyButton();
		gmc = new GroupManagementController(this);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init
	 * (org.eclipse.ui.IWorkbench)
	 */
	@Override
	public void init(IWorkbench workbench) {
		setDescription
			("Registered EMF refactorings can be arranged by groups.");
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.preference.PreferencePage#createContents
	 * (org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected Control createContents(Composite parent) {
		parent.setLayout(new GridLayout(1, false));
		setMenuWidget(parent);
		enableMenuWidget(true);
		if (EmfRefactorConfig.getInstance().getRefactorings().isEmpty()){
			setErrorMessage("There is no registered " +
									"EMF model refactoring available.");
			enableMenuWidget(false);
		}
		setSelectionWidget(parent);
		enableSelectionWidget(false);
		setNameWidget(parent);
		enableNameWidget(false);
		return menuWidget;
	}
	
	private void setNameWidget(Composite parent) {
		nameWidget = new Composite(parent, SWT.BORDER);
		nameWidget.setLayout(new GridLayout(2, false));		
		Composite interactionWidget = new Composite(nameWidget, SWT.NONE);
		interactionWidget.setLayout(new FillLayout(SWT.VERTICAL));
		Label label = new Label(interactionWidget, SWT.NONE);
		label.setText("Please enter a name of the refactoring group: ");
		textName = new Text(interactionWidget, SWT.NONE);
		Composite buttonWidget = new Composite(nameWidget, SWT.NONE);
		buttonWidget.setLayoutData(new GridData
									(SWT.RIGHT, SWT.NONE, false, false));
		buttonWidget.setLayout(new RowLayout(SWT.HORIZONTAL));
		nameOkButton = new Button(buttonWidget,SWT.PUSH);
		nameOkButton.setText("OK");
		nameOkButton.setLayoutData(new RowData(100, 25));
		nameOkButton.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					delegateBack();
				}
			});
		nameCancelButton = new Button(buttonWidget,SWT.PUSH);
		nameCancelButton.setText("Cancel");
		nameCancelButton.setLayoutData(new RowData(100, 25));
		nameCancelButton.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					reset();
				}
			});	
	}

	private void setSelectionWidget(Composite parent) {
		selectionWidget = new Composite(parent, SWT.BORDER);
		selectionWidget.setLayout(new GridLayout(2, false));
		
		Composite interactionWidget = new Composite(selectionWidget, SWT.NONE);
		interactionWidget.setLayout(new FillLayout(SWT.VERTICAL));
		Label label = new Label(interactionWidget, SWT.NONE);
		label.setText("Please select an existing refactoring group:      ");
		combo = new Combo(interactionWidget, SWT.READ_ONLY);
		
		Composite selectionButtonWidget = 
									new Composite(selectionWidget, SWT.NONE);
		selectionButtonWidget.setLayoutData
							(new GridData(SWT.RIGHT, SWT.NONE, false, false));
		selectionButtonWidget.setLayout(new RowLayout(SWT.HORIZONTAL));
		selectionOkButton = new Button(selectionButtonWidget,SWT.PUSH);
		selectionOkButton.setText("OK");
		selectionOkButton.setLayoutData(new RowData(100, 25));
		selectionOkButton.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					gmc.continueBeyondSelection
									(combo.getItem(combo.getSelectionIndex()));
				}
			});
		selectionCancelButton = new Button(selectionButtonWidget,SWT.PUSH);
		selectionCancelButton.setText("Cancel");
		selectionCancelButton.setLayoutData(new RowData(100, 25));
		selectionCancelButton.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					reset();
				}
			});
	}
	
	public void reset() {
		combo.removeAll();
		textName.setText("");
		enableMenuWidget(true);
		enableSelectionWidget(false);
		enableNameWidget(false);
	}
	
	protected void delegateBack() {
		String name = textName.getText().trim();
		if (name.isEmpty()){
			MessageDialog.openError(this.getShell(), "EMF Refactor", 
							"Please enter a name of the refactoring group!");
		} else {
			if (gmc.existsGroup(name)){
				MessageDialog.openError(this.getShell(), "EMF Refactor", 
							"There is already a group named '" + name + "'!");
			} else {
				gmc.setGroupName(name);
				gmc.continueBeyondNameDialog();
			}
		}
	}

	private void setMenuWidget(Composite parent) {
		menuWidget = new Composite(parent, SWT.BORDER);
		menuWidget.setLayout(new GridLayout(2, false));
		
		Composite modusWidget = new Composite(menuWidget, SWT.NONE);
		modusWidget.setLayout(new GridLayout(1, true));
		addButton = new Button(modusWidget, SWT.RADIO);
	    addButton.setText("Add new Refactoring Group");
	    updateButton = new Button(modusWidget, SWT.RADIO);
	    updateButton.setText("Update existing Refactoring Group");
	    showButton = new Button(modusWidget, SWT.RADIO);
	    showButton.setText("Show existing Refactoring Group");
	    deleteButton = new Button(modusWidget, SWT.RADIO);
	    deleteButton.setText("Delete existing Refactoring Group");
	    renameButton = new Button(modusWidget, SWT.RADIO);
	    renameButton.setText("Rename existing Refactoring Group");
	    addButton.setSelection(true);
		
	    Composite buttonWidget = new Composite(menuWidget, SWT.NONE);
		buttonWidget.setLayout(new RowLayout(SWT.VERTICAL));
		Button dummyButton1 = new Button(buttonWidget,SWT.PUSH);
		dummyButton1.setVisible(false);
		Button dummyButton2 = new Button(buttonWidget,SWT.PUSH);
		dummyButton2.setVisible(false);
		Button dummyButton3 = new Button(buttonWidget,SWT.PUSH);
		dummyButton3.setVisible(false);
		menuOkButton = new Button(buttonWidget,SWT.PUSH);
		menuOkButton.setText("OK");
		menuOkButton.setLayoutData(new RowData(100, 25));
		menuOkButton.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					gmc.startModus(getModusFromButtons());
				}
			});
	}
	
	private void enableMenuWidget(boolean b) {
		addButton.setEnabled(b);
		updateButton.setEnabled(b);
		showButton.setEnabled(b);
		deleteButton.setEnabled(b);
		renameButton.setEnabled(b);
		menuOkButton.setEnabled(b);
	}
	
	private void enableSelectionWidget(boolean b) {
		selectionCancelButton.setEnabled(b);
		selectionOkButton.setEnabled(b);
		combo.setEnabled(b);
	}
	
	private void enableNameWidget(boolean b) {
		nameCancelButton.setEnabled(b);
		nameOkButton.setEnabled(b);
		textName.setEnabled(b);
	}
	
	private GroupManagementModus getModusFromButtons() {
		GroupManagementModus modus = null;
		if (this.addButton.getSelection()) modus = 
							GroupManagementModus.NEWGROUP;
		if (this.updateButton.getSelection()) modus = 
							GroupManagementModus.UPDATEGROUP;
		if (this.showButton.getSelection()) modus = 
							GroupManagementModus.SHOWGROUP;
		if (this.deleteButton.getSelection()) modus = 
							GroupManagementModus.DELETEGROUP;
		if (this.renameButton.getSelection()) modus = 
							GroupManagementModus.RENAMEGROUP;
		return modus;
	}

	public void startSelection() {
		enableMenuWidget(false);
		fillCombo();
		enableSelectionWidget(true);
		if (combo.getItemCount() == 0) {
			selectionOkButton.setEnabled(false);
		} else {
			combo.select(0);
		}
	}
	
	public void stopSelection() {
		enableSelectionWidget(false);
	}

	private void fillCombo() {
		for (Group group : gmc.getGroupManagement().getGroups()){
			String name = group.getName();
			if ( ! name.equals(gmc.DEFAULTGROUP)){
				combo.add(name);
			}
			if (gmc.getModus().equals(GroupManagementModus.SHOWGROUP) && 
					name.equals(gmc.DEFAULTGROUP)) {
				combo.add(name);
			}
		}
	}

	public void startNaming(String groupName) {
		enableMenuWidget(false);
		enableSelectionWidget(false);
		textName.setText(groupName);
		enableNameWidget(true);
	}
}
