/*******************************************************************************
 * Copyright (c) Philipps University of Marburg. All rights reserved. 
 * This program and the accompanying materials are made 
 * available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Philipps University of Marburg - initial API and implementation
 *******************************************************************************/
package org.eclipse.emf.refactor.common.groupmanagement.ui;

import org.eclipse.emf.refactor.common.groupmanagement.EmfRefactoringEntry;
import org.eclipse.emf.refactor.common.groupmanagement.Group;
import org.eclipse.emf.refactor.common.groupmanagement.GroupManagementController;
import org.eclipse.emf.refactor.common.groupmanagement.GroupmanagementFactory;
import org.eclipse.emf.refactor.common.groupmanagement.MetaModelGroup;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;

/**
 * Dialog for assigning existing EMF model refactorings to a specific
 * model refactoring group.
 * @generated NOT
 * @author Thorsten Arendt
 */
public class GroupManagementRefactoringDialog extends Shell {
	
	/**
	 * Controller of the group management process.
	 */
	private GroupManagementController gmController;
	
	/**
	 * Actual EMF model refactoring group.
	 */
	private Group group;
	
	private Composite groupNameWidget, refactoringsWidget, buttonWidget;
	private TabFolder tabFolder;
	private Button okButton;
	
	/**
	 * Contructor using the controller of the group management process.
	 * @param groupManagementController Controller of the group management 
	 * process.
	 */
	public GroupManagementRefactoringDialog
					(GroupManagementController groupManagementController) {
		super(groupManagementController.getGroupManagementPreferencePage()
																.getShell());
		this.gmController = groupManagementController;
		this.init();
	}
	
	/**
	 * Contructor using the controller of the group management process, the
	 * actual refactoring group and a flag for disabling the ok button.
	 * @param groupManagementController Controller of the group management 
	 * process.
	 * @param group Actual refactoring group.
	 * @param disableOkButton Flag for disabling the ok button.
	 */
	public GroupManagementRefactoringDialog(
			GroupManagementController groupManagementController, 
			Group group, boolean disableOkButton) {
		super(groupManagementController.getGroupManagementPreferencePage().
																getShell());
		this.gmController = groupManagementController;
		this.init(group, disableOkButton);
	}
	
	private void init(Group group, boolean disableOkButton) {
		this.setText("EMF Refactor - Customize Refactorings");
		this.setLayout(new GridLayout(1, false));
		initGroupNameWidget();
		initButtonWidget();
		this.okButton.setEnabled(disableOkButton);
		initRefactoringsWidget();
		this.setEntries(group);
		this.pack();
		this.open();
		updateSizes();
		this.redraw();
		this.update();
		while (! this.isDisposed()){
			if (! Display.getDefault().readAndDispatch()) {
				Display.getDefault().sleep();
			}
		}	
	}
	
	private void init() {
		this.setText("EMF Refactor - Customize Refactorings");
		this.setLayout(new GridLayout(1, false));
		initGroupNameWidget();
		initButtonWidget();
		initRefactoringsWidget();
		this.pack();
		this.open();
		updateSizes();
		this.redraw();
		this.update();
		while (! this.isDisposed()){
			if (! Display.getDefault().readAndDispatch()) {
				Display.getDefault().sleep();
			}
		}	
	}
	
	private void initButtonWidget() {
		buttonWidget = new Composite(this, SWT.NONE);
		buttonWidget.setLayoutData
						(new GridData(SWT.RIGHT, SWT.NONE, false, false));
		buttonWidget.setLayout(new RowLayout(SWT.HORIZONTAL));
		okButton = new Button(buttonWidget,SWT.PUSH);
		okButton.setText("OK");
		okButton.setLayoutData(new RowData(100, 25));
		okButton.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					delegateBack();
				}
			});
		Button cancelButton = new Button(buttonWidget,SWT.PUSH);
		cancelButton.setText("Cancel");
		cancelButton.setLayoutData(new RowData(100, 25));
		cancelButton.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					dispose();
					gmController.getGroupManagementPreferencePage().reset();
				}
			});
	}
	
	private void delegateBack(){
		if (noSelection()){
			MessageDialog.openError(null, "EMF Refactor", 
					"There is no refactoring selected for this group!");
		} else {
			if (noAlias()){
				MessageDialog.openError(null, "EMF Refactor", 
						"There is an empty alias name! Please type in.");
			} else {
				composeGroup();
				gmController.continueBeyondRefactoringDialog(this.group);
				dispose();
				MessageDialog.openInformation(null, "EMF Refactor", 
					"New Group '" + gmController.getGroupName() + "' saved.");
			}
		}
	}
	
	private void composeGroup(){
		GroupmanagementFactory groupmanagementFactory = 
											GroupmanagementFactory.eINSTANCE;
		group = groupmanagementFactory.createGroup();
		group.setName(gmController.getGroupName());
		for (TabItem tabItem : tabFolder.getItems()){
			MetaModelGroup mmGroup = 
								groupmanagementFactory.createMetaModelGroup();
			mmGroup.setNsUri(tabItem.getText());
			Table table = (Table) tabItem.getControl();
			for (TableItem tableItem : table.getItems()){
				if (tableItem.getChecked()){
					EmfRefactoringEntry entry = 
							groupmanagementFactory.createEmfRefactoringEntry();
					entry.setId(tableItem.getText(1));
					entry.setName(tableItem.getText(2));
					entry.setAlias(tableItem.getText(3));
					entry.setMmgroup(mmGroup);
					mmGroup.getEntries().add(entry);
				}
			}
			if (! mmGroup.getEntries().isEmpty()){
				group.getMmGroups().add(mmGroup);
			}
		}	
	}
	
	private boolean noAlias(){
		for (TabItem tabItem : tabFolder.getItems()){
			Table table = (Table) tabItem.getControl();
			for (TableItem tableItem : table.getItems()){
				if (tableItem.getChecked()){
					if (null == tableItem.getText(3) || 
								tableItem.getText(3).trim().isEmpty()) {
						return true;
					}
				}
			}
		}
		return false;
	}
	
	private boolean noSelection(){
		for (TabItem tabItem : tabFolder.getItems()){
			Table table = (Table) tabItem.getControl();
			for (TableItem tableItem : table.getItems()){
				if (tableItem.getChecked()){
					return false;
				}
			}
		}
		return true;
	}
	
	private void updateSizes() {
		int x = this.refactoringsWidget.getSize().x;
		int ygw = this.groupNameWidget.getSize().y;
		int ybw = this.buttonWidget.getSize().y;
		int yrw = 400;
		int margin = 20;
		this.setSize(x + margin, ygw + ybw + yrw + 3* margin);
		this.buttonWidget.setSize(x, ybw);
		this.refactoringsWidget.setSize(x, yrw);
		this.tabFolder.setSize(x - margin, yrw - margin);
	}
	
	private void initRefactoringsWidget() {
		refactoringsWidget = new Composite(this, SWT.BORDER);
		refactoringsWidget.setLayout(new RowLayout(SWT.VERTICAL));
		tabFolder = new TabFolder(refactoringsWidget, SWT.NONE);		
		Group defaultGroup = gmController.getDefaultGroup();
		for (MetaModelGroup mmGroup : defaultGroup.getMmGroups()){
			TabItem tabItem = new TabItem(tabFolder, SWT.NONE);
		    tabItem.setText(mmGroup.getNsUri());
		    tabItem.setToolTipText(mmGroup.getNsUri());
		    final Table table = new Table(tabFolder, 
		    		SWT.CHECK  | SWT.SINGLE | SWT.FULL_SELECTION 
		    		| SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL);
			table.setHeaderVisible(true);
			TableColumn col;
			col = new TableColumn(table, SWT.LEFT | SWT.READ_ONLY);
			col.setText("sel");
			col.setWidth(30);
			col = new TableColumn(table, SWT.LEFT | SWT.READ_ONLY);
			col.setText("Refactoring id");
			col.setWidth(200);
			col = new TableColumn(table, SWT.LEFT | SWT.READ_ONLY);
			col.setText("Refactoring name");
			col.setWidth(200);
			col = new TableColumn(table, SWT.LEFT );
			col.setText("alias name");
			col.setWidth(200);
			tabItem.setControl(table);
			final TableEditor editor = new TableEditor(table);
			editor.horizontalAlignment = SWT.LEFT;
			editor.grabHorizontal = true;
			editor.minimumWidth = 200;
			final int EDITABLECOLUMN = 3;
			table.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					Control oldEditor = editor.getEditor();
					if (oldEditor != null) oldEditor.dispose();
					TableItem item = (TableItem)e.item;
					if (item == null) return;
					Text newEditor = new Text(table, SWT.NONE);
					newEditor.setText(item.getText(EDITABLECOLUMN));
					newEditor.addModifyListener(new ModifyListener() {
							public void modifyText(ModifyEvent me) {
									Text text = (Text)editor.getEditor();
									editor.getItem().setText
											(EDITABLECOLUMN, text.getText());
							}
					});
					newEditor.selectAll();
					newEditor.setFocus();
					editor.setEditor(newEditor, item, EDITABLECOLUMN);
				}
			});
			for (EmfRefactoringEntry entry : mmGroup.getEntries()){
				TableItem item = new TableItem (table, SWT.NONE);
				item.setText (1, entry.getId());
				item.setText (2, entry.getName());
				item.setText (3, entry.getAlias());
			}
		}
	}
	
	private void initGroupNameWidget() {
		groupNameWidget = new Composite(this, SWT.NONE);
		groupNameWidget.setLayout(new GridLayout(2, true));
		Label groupLabel = new Label(groupNameWidget, SWT.NONE);
		groupLabel.setText("Refactoring Group Name: ");		
		Label groupNameLabel = new Label(groupNameWidget, SWT.NONE);
		groupNameLabel.setText(gmController.getGroupName());
	}
	
	protected void checkSubclass() { }
	
	public void setEntries(Group group) {
		for (TabItem tabItem : tabFolder.getItems()){
			Table table = (Table) tabItem.getControl();
			for (TableItem tableItem : table.getItems()){
				String alias = getAliasWhenValidRefactoring
								(group, tabItem.getText(), tableItem.getText(1), 
										tableItem.getText(2));
				if (! alias.isEmpty()){
					tableItem.setChecked(true);		
					tableItem.setText (3, alias);
				}
			}
		}
	}
	
	private String getAliasWhenValidRefactoring
						(Group group, String uri, String id, String name) {
		String alias = "";
		for (MetaModelGroup mmGroup : group.getMmGroups()) {
			if (mmGroup.getNsUri().equals(uri)) {
				for (EmfRefactoringEntry entry : mmGroup.getEntries()) {
					if (entry.getId().equals(id) 
										&& entry.getName().equals(name)) {
						alias = entry.getAlias();
						break;
					}
				}
			}
		}
		return alias;
	}

}
