/*******************************************************************************
 * Copyright (c) Philipps University of Marburg. All rights reserved. 
 * This program and the accompanying materials are made 
 * available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Philipps University of Marburg - initial API and implementation
 *******************************************************************************/
package org.eclipse.emf.refactor.common.projectgroupmapping;

import java.util.ArrayList;
import java.util.List;

/**
 * Class for mapping a project to several EMF model refactoring groups.
 * @generated NOT
 * @author Thorsten Arendt
 */
public class ProjectGroupMapping {
	
	/**
	 * Name of the project to be mapped.
	 */
	private String projectName;
	
	/**
	 * List of group names that are mapped to the project.
	 */
	private List<String> groupNames;
	
	/**
	 * Default constructor using the name of the project to be mapped.
	 * @param projectName Name of the project to be mapped.
	 */
	public ProjectGroupMapping(String projectName){
		this.projectName = projectName;
		this.groupNames = new ArrayList<String>();
	}
	
	/**
	 * Gets the name of the project to be mapped.
	 * @return Name of the project to be mapped.
	 */
	public String getProjectName(){
		return this.projectName;
	}
	
	/**
	 * Gets the names of the groups mapped to the project.
	 * @return List of group names that are mapped to the project.
	 */
	public List<String> getGroupNames(){
		return this.groupNames;
	}
	
	/**
	 * Adds a group name to the mapped list of mapped group names.
	 * @param groupName Name of the group to be added to the list 
	 * of mapped group names.
	 */
	public void addGroupName(String groupName){
		this.groupNames.add(groupName);
	}	 

}
