/*******************************************************************************
 * Copyright (c) Philipps University of Marburg. All rights reserved. 
 * This program and the accompanying materials are made 
 * available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Philipps University of Marburg - initial API and implementation
 *******************************************************************************/
package org.eclipse.emf.refactor.common.projectgroupmapping.ui;

import org.eclipse.core.resources.IProject;
import org.eclipse.emf.refactor.common.config.EmfRefactorConfig;
import org.eclipse.emf.refactor.common.groupmanagement.Group;
import org.eclipse.emf.refactor.common.projectgroupmapping.ProjectGroupMapping;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.PropertyPage;

/**
 * PropertyPage for mapping a project to several EMF model 
 * refactoring groups.
 * @generated NOT
 * @author Thorsten Arendt
 */
public class ProjectGroupMappingPropertyPage extends PropertyPage {
	
	/**
	 * Configuration object of Emf Refactor.
	 */
	private EmfRefactorConfig container = EmfRefactorConfig.getInstance();
	
	/**
	 * Actual project that belongs to the PropertyPage.
	 */
	private IProject project;
	
	private Table table;

	/**
	 * @see org.eclipse.jface.preference.PreferencePage#createContents
	 * (org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected Control createContents(Composite parent) {
		initProject();
		Composite composite = new Composite(parent, SWT.NONE);
		GridLayout gl = new GridLayout();
		gl.verticalSpacing = 20;
		gl.marginHeight = 20;
		gl.marginWidth = 20;
		gl.numColumns = 1;
		composite.setLayout(gl);
		createHeader(composite);	
		createGroupEntries(composite);	
		return composite;
	}
	
	/**
	 * Initializes the actual project that belongs to the PropertyPage.
	 */
	private void initProject() {
		if (this.getElement() instanceof IJavaProject) {
			project = ((IJavaProject) this.getElement()).getProject();
		} else {
			project = ((IProject) this.getElement());
		}
	}

	private void createHeader(Composite composite) {
		Text text = new Text(composite, SWT.LEFT);
		text.setBackground(composite.getBackground());
		text.setText
			("Please select EMF model refactoring groups from " +
					"the list below to be supported by the project.");
	}
	
	private void createGroupEntries(Composite composite) {
		table = new Table(composite, 
					SWT.CHECK  | SWT.SINGLE | SWT.FULL_SELECTION 
					| SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL);
		table.setHeaderVisible(true);
		TableColumn col;
		col = new TableColumn(table, SWT.LEFT | SWT.READ_ONLY);
		col.setText("Selected");
		col.setWidth(60);
		col = new TableColumn(table, SWT.LEFT | SWT.READ_ONLY);
		col.setText("Refactoring group name");
		col.setWidth(360);		
		this.container.loadProjectGroupMappings(project);
		this.container.synchronizeGroupMappings();
		this.container.saveProjectGroupMapping(project);
		for (Group group : this.container.getGroupManagement().getGroups()) {	
			TableItem item = new TableItem (table, SWT.NONE);
			item.setText (1, group.getName());
		}
		markGroups(); 		
	}
	
	private void markGroups() {
		for (Group group : this.container.getGroupManagement().getGroups()){
			if (this.container.isValidGroup(group.getName())){
				int groupIndex = getIndexOfGroup(group.getName());
				table.getItems()[groupIndex].setChecked(true);
			}
		}
	}
	
	private int getIndexOfGroup(String groupName) {
		int index = -1;
		for (TableItem item : table.getItems()){
			if (item.getText(1).equals(groupName)){
				index = table.indexOf(item);
				break;
			}
		}
		return index;
	}
	
	/**
	 * @see org.eclipse.jface.preference.PreferencePage#performOk()
	 */
	@Override
	public boolean performOk(){
		container.setProjectGroupMapping(createProjectGroupMapping());
		container.saveProjectGroupMapping(project);
		return true;
	}
	
	private ProjectGroupMapping createProjectGroupMapping() {
		ProjectGroupMapping pgm = 
			new ProjectGroupMapping((project).getName());
		for (TableItem item : table.getItems()){
			if (item.getChecked()){
				pgm.addGroupName(item.getText(1));
			}
		}	
		return pgm;
	}

	/**
	 * @see org.eclipse.jface.preference.PreferencePage#performApply()
	 */
	@Override
	protected void performApply(){
		performOk();
	}
	
	/**
	 * @see org.eclipse.jface.preference.PreferencePage#performDefaults()
	 */
	@Override
	protected void performDefaults(){
		for (TableItem item : table.getItems()){
			item.setChecked(true);
		}
	}
	

}
