/*******************************************************************************
 * Copyright (c) Philipps University of Marburg. All rights reserved. 
 * This program and the accompanying materials are made 
 * available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Philipps University of Marburg - initial API and implementation
 *******************************************************************************/
package org.eclipse.emf.refactor.henshin.generator;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.jar.Attributes;
import java.util.jar.Manifest;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.emf.codegen.jet.JETEmitter;
import org.eclipse.emf.codegen.jet.JETException;
import org.eclipse.emf.refactor.generator.RefactoringGenerator;
import org.eclipse.emf.refactor.henshin.Activator;
import org.eclipse.jdt.core.JavaCore;
import org.osgi.framework.Bundle;

/**
 * Main class for generating model refactoring code out of an object of class
 * RefactoringConfigHenshin using JET.
 * @generated NOT
 * @author Thorsten Arendt
 */
public class RefactoringGeneratorHenshin extends RefactoringGenerator {
	
	private final String REFACTORHENSHIN = 
										"org.eclipse.emf.refactor.henshin";	
	private final String REFACTORINGWIZARDHENSHIN = 
										"RefactoringWizardHenshin";	
	private final String REFACTORINGWIZARDPAGEHENSHIN = 
										"RefactoringWizardPageHenshin";
	private final String REFACTORINGGUIHANDLERHENSHIN = 
										"RefactoringGuiHandlerHenshin";
	private final String REFACTORINGCONTROLLERHENSHIN = 
										"RefactoringControllerHenshin";
	private final String REFACTORINGDATAMANAGEMENTHENSHIN = 
										"RefactoringDataManagementHenshin";
	private final String REFACTORINGINFORMATIONHENSHIN = 
										"RefactoringInformationHenshin";
	private final String REFACTORINGINFORMATION = 
										"RefactoringInformation";
	private final String HENSHINMODEL = "org.eclipse.emf.henshin.model";
	private final String ECOREXMI = "org.eclipse.emf.ecore.xmi";
	protected final String REFACTORINGTESTHENSHIN = "RefactoringTestHenshin";
	protected final String CONFIGHENSHIN = "ConfigHenshin";
	
	/**
	 * Configuration data used for generating model refactoring code.
	 */
	private RefactoringConfigHenshin configHenshin;
	
	/**
	 * Default constructor using configuration data.
	 * @param configHenshin Configuration data used for generating model 
	 * refactoring code.
	 */
	public RefactoringGeneratorHenshin
						(RefactoringConfigHenshin configHenshin) {
		super(configHenshin);
		this.configHenshin = configHenshin;
		this.templateDirectory = this.setNewTemplateDirectory();
		this.addHenshinClassPathEntry();
	}
	
	/**
	 * @see org.eclipse.emf.refactor.generator.RefactoringGenerator#
	 * run(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void run(IProgressMonitor monitor){
		deleteJetEmittersProject(monitor);
		updatePluginXml();
		updateDependencies();
		setSingletonDirective();
		createTestFolders(monitor);
		generateCode(monitor);
	}
	
	/**
	 * Sets the singleton directive of the plugin to true.
	 */
	protected void setSingletonDirective() {
		IProject project = ResourcesPlugin.getWorkspace().getRoot()
							.getProject(this.config.getProjectName());
		try {
			String fileName = project.getLocation()
					.append("/META-INF/MANIFEST.MF")
					.toOSString();
			FileInputStream is = new FileInputStream(fileName);
			Manifest mf = new Manifest(is);
			Attributes att = mf.getMainAttributes();
			String value = att.getValue(BUNDLESYMBOLICNAME);
			if (! value.contains(SINGLETONTRUE))
				value = value + ";" +  SINGLETONTRUE;
			att.putValue(BUNDLESYMBOLICNAME, value);
			FileOutputStream out = new FileOutputStream(fileName);  
			mf.write(out); 
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}  catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Updates the plugin-dependencies (Require-Bundle) of the
	 * target project. Adds dependencies to org.eclipse.emf.ecore,
	 * org.eclipse.emf.ecore.change, org.eclipse.ltk.core.refactoring,
	 * org.eclipse.ltk.ui.refactoring, org.eclipse.emf.refactor.common,
	 * org.eclipse.emf.refactor.runtime, and org.eclipse.emf.refactor.henshin
	 */
	private void updateDependencies() {		
		IProject project = ResourcesPlugin.getWorkspace().getRoot()
						.getProject(this.configHenshin.getProjectName());
		try {
			String fileName = project.getLocation()
	         							.append("/META-INF/MANIFEST.MF")
	         							.toOSString();
			FileInputStream is = new FileInputStream(fileName);			
			Manifest mf = new Manifest(is);
			Attributes att = mf.getMainAttributes();
			String value = att.getValue(REQUIREBUNDLE);
			if (! value.contains(EXPRESSIONS)) 
			    value = value + "," +  EXPRESSIONS;
			if (! value.contains(ECORE))
				value = value + "," +  ECORE;
			if (! value.contains(ECORECHANGE))
				value = value + "," +  ECORECHANGE;
			if (! value.contains(ECOREXMI))
			    value = value + "," +  ECOREXMI;
			if (! value.contains(HENSHINMODEL))
				value = value + "," +  HENSHINMODEL;
			if (! value.contains(LTKCORE))
				value = value + "," +  LTKCORE;
			if (! value.contains(LTKUI))
				value = value + "," +  LTKUI;
			if (! value.contains(REFACTORCOMMON)) 
				value = value + "," +  REFACTORCOMMON;
			if (! value.contains(REFACTORRUNTIME)) 
				value = value + "," +  REFACTORRUNTIME;
			if (! value.contains(REFACTORHENSHIN))
				value = value + "," +  REFACTORHENSHIN;
			if (! value.contains(JUNIT)) 
			    value = value + "," +  JUNIT + ";bundle-version=\"4.8.1\"";
			if (! value.contains(this.configHenshin.getSelectedEObjectJar()))
				value = value + "," 
						+  this.configHenshin.getSelectedEObjectJar();
			att.putValue(REQUIREBUNDLE, value);
			FileOutputStream out = new FileOutputStream(fileName); 
			mf.write(out); 
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}  catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Method for generating model refactoring code. This includes
	 * "RefactoringWizard.java", "RactoringWizardPage.java",
	 * "RefactoringGuiHandler.java", "RefactoringController.java", 
	 * "RefactoringDataManagement.java", and "RefactoringInformation.java".
	 * @param monitor Object for monitoring the code generation.
	 */
	private void generateCode(IProgressMonitor monitor) {
		String generatedCode = "";
		try {
			generatedCode = 
				this.generateCode(monitor, REFACTORINGWIZARDHENSHIN);
			this.save(monitor, generatedCode, REFACTORINGWIZARD);
			generatedCode = 
				this.generateCode(monitor, REFACTORINGWIZARDPAGEHENSHIN);
			this.save(monitor, generatedCode, REFACTORINGWIZARDPAGE);
			generatedCode = 
				this.generateCode(monitor, REFACTORINGGUIHANDLERHENSHIN);
			this.save(monitor, generatedCode, REFACTORINGGUIHANDLER);
			generatedCode = 
				this.generateCode(monitor, REFACTORINGCONTROLLERHENSHIN);
			this.save(monitor, generatedCode, REFACTORINGCONTROLLER);
			generatedCode = 
				this.generateCode(monitor, REFACTORINGDATAMANAGEMENTHENSHIN);
			this.save(monitor, generatedCode, REFACTORINGDATAMANAGEMENT);
			generatedCode = 
				this.generateCode(monitor, REFACTORINGINFORMATIONHENSHIN);
			this.save(monitor, generatedCode, REFACTORINGINFORMATION);
			generatedCode =
				this.generateCode(monitor, REFACTORINGTESTHENSHIN);
			this.save(monitor, generatedCode, REFACTORINGTEST);
		} catch (JETException e) {
			e.printStackTrace();
		} catch (CoreException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Generates code out of the template with the given name.
	 * @param monitor Object monitoring the code generation.
	 * @param template Name of the template used by JET.
	 * @return Generated code as String.
	 */
	private String generateCode(IProgressMonitor monitor, String template) {
		String templatePath = this.templateDirectory + template + JAVAJET;
		ClassLoader classLoader = getClass().getClassLoader();
		this.jetEmitter = new JETEmitter(templatePath, classLoader);
		this.jetEmitter.getClasspathEntries().addAll(classpathEntries);
		String result = "";
		IProgressMonitor sub = new SubProgressMonitor(monitor, 1);
		try {
			result = 
				jetEmitter.generate(sub, new Object[] { this.configHenshin });
		} catch (JETException e) {
			e.printStackTrace();
		}
		return result;
	}
	
	private String setNewTemplateDirectory() {
		String td = "";
		final Bundle bundle = Activator.getDefault().getBundle();
		try {
			td = FileLocator.toFileURL(bundle.getEntry(TEMPLATES)).getFile();
		} catch (final IOException e) {
			e.printStackTrace();
		}
		return td;
	}
	
	private void addHenshinClassPathEntry() {
		Bundle bundle = Platform.getBundle(Activator.PLUGIN_ID);
		String version = (String) bundle.getHeaders().get(BUNDLEVERSION);
		classpathEntries.add(JavaCore.newLibraryEntry(new Path(PLUGINSPATH
				+ Activator.PLUGIN_ID + "_" + version + ".jar"), null, null));
	}	
	
	/**
	 * Creates the set of test folders for the generated model refactoring
	 * as well as the contained config.xml files.
	 * @param monitor Object monitoring the code generation.
	 */
	protected void createTestFolders(IProgressMonitor monitor) {
		String packageName = this.config.getPackageName();
		packageName = packageName.substring(0, packageName.lastIndexOf('.'));
		String path = Platform.getInstanceLocation()
											.getURL().getPath() + packageName;
		if (this.config.getNumberOfTests() > 0) {
			try {
				File testFolder = new File(path + "/tests/" + 
										this.config.getRefactoringId() + "/");
				testFolder.mkdirs();	
				if (testFolder.exists() && testFolder.isDirectory()){
					String testfolderprefix = 
							testFolder.getCanonicalPath() + "/test_";
					for (int i = 1; i <= this.config.getNumberOfTests(); i++) {
						File specifictestfolder = new File(testfolderprefix 
											+ String.format("%03d", i) + "/");
						if (!specifictestfolder.exists() 
								|| !specifictestfolder.isDirectory()) {
							specifictestfolder.mkdir();
							createConfigXml(monitor, specifictestfolder);
						}
					}
				}					
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * Creates the config.xml files within the test folders for the 
	 * generated model refactoring.
	 * @param monitor Object monitoring the code generation.
	 * @param xmlFolder File object containing the folder in which 
	 * the config should be saved. 
	 */
	protected void createConfigXml(IProgressMonitor monitor, File xmlFolder) {
		String generatedCode = this.generateCode(monitor, CONFIGHENSHIN);
		try {
			File configXml = new File
							(xmlFolder.getCanonicalPath() + "/config.xml");
			if (!configXml.exists() || !configXml.isFile()) {
				this.saveXml(monitor, generatedCode, 
									configXml.getCanonicalPath().toString());
			}
		} catch (IOException e) {
			e.printStackTrace();
		} catch (JETException e) {
			e.printStackTrace();
		} catch (CoreException e) {
			e.printStackTrace();
		}
	}

}
