package org.eclipse.emf.refactor.runtime.test;

import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.emf.common.command.AbstractCommand;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.refactor.runtime.ltk.LtkEmfRefactoringProcessorAdapter;
import org.eclipse.ltk.core.refactoring.participants.RefactoringProcessor;

public class EmfTestRefactoringCommand extends AbstractCommand {

	private final EObject root;
	private final LtkEmfRefactoringProcessorAdapter processor;

	/**
	 * Constructor.
	 * @param name Name of the refactoring to run.
	 * @param newProcessor RefactoringProcessor to be used by the command.
	 * @param rootObject Root EObject to run the refactoring on.
	 */
	public EmfTestRefactoringCommand(
			final String name,
			final RefactoringProcessor newProcessor,
			final EObject rootObject) {
		super(name);
		this.root = rootObject;
		this.processor =
			(LtkEmfRefactoringProcessorAdapter) newProcessor;
	}

	@Override
	public final void execute() {
		try {
			// check whether all conditions are met
			if (this.processor.checkConditions()) {
				// apply refactoring
				this
				.processor
				.createChange(new NullProgressMonitor())
						.perform(new NullProgressMonitor());
			} else {
				// TODO real error handling that gets displayed
				System.out.println("Checks failed");
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Returns to the root EObject.
	 * @return Root EObject 
	 */
	protected final EObject getRoot() {
		return root;
	}

	@Override
	public void redo() {
		// do nothing
	}

	@Override
	public void undo() {
		// do nothing
	}

	@Override
	public final boolean canExecute() {
		return true;
	}

	@Override
	public final boolean canUndo() {
		return false;
	}

}